#! /usr/bin/python

import os
import sys
import string
import glob


orig_res = [300, 200, 630]
#orig_res = [600, 400, 630]

err_func = 'avgabs'

low_format = '%s.d%d.fdl'
err_format = '%s.%s.d%d.e%d.fdl'
dest_format = 'silo/ar_ud_r%d_%s_e%.2f/'

error_offset = 4


geometry_file = 'thm-20070323-j13c.630x200x300.grid'
#geometry_file = 'thm-20070323-j13c.630x400x600.grid'

fdl_contents = '''<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE FileDescriptor PUBLIC "-//SDB//DTD//ED" "fdl.dtd">
<FileDescriptor fileName= "%s"  fileType="binary" recordSize="4" byteOrder="little">
    <Bounds lower= "0"    upper= "%d"/>
    <Bounds lower= "0"    upper= "%d"/>
    <Bounds lower= "0"    upper= "%d"/>
    <Field fieldName= "temp"   fieldType= "float"/>
</FileDescriptor>
'''

mrdl_contents = '''<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE mr_descriptor PUBLIC "-//SDB//DTD//EN"  "http://www.cs.unh.edu/~svohr/DTD/mrdl.dtd">
<mr_descriptor levels="4" offset="%d" coords="uniform">
    <level id="0">
        <source type="data" resolution="0" desc="%%s"/>
    </level>
    <level id="1">
        <source type="data" resolution="1" desc="%%s"/>
        <source type="error" resolution="%d" desc="%%s"/>
    </level>
    <level id="2">
            <source type="data" resolution="2" desc="%%s"/>
            <source type="error" resolution="%d" desc="%%s"/>
    </level>
    <level id="3">
         <source type="data" resolution="3" desc="%%s"/>
         <source type="error" resolution="%d" desc="%%s"/>
    </level>
</mr_descriptor>
'''


def dump_fdl (filename):
    fdl_filename = filename + '.fdl'
    fdl_out = open(fdl_filename, 'w')
    fdl_out.write ( fdl_contents % 
                    (filename, orig_res[0]-1, orig_res[1]-1, orig_res[2]-1) )
    return fdl_filename

def dump_mrdl (filename, data_fdls, error_fdls):
    mrdl_filename = filename + '.mrdl'
    fdl_filename = filename + '.fdl'
    mrdl_out = open(mrdl_filename, 'w')
    contents = mrdl_contents % ( 
        error_offset, 1+error_offset, 2+error_offset, 3+error_offset )
    mrdl_out.write ( contents % (   fdl_filename, 
                                    data_fdls[0], error_fdls[0],
                                    data_fdls[1], error_fdls[1],
                                    data_fdls[2], error_fdls[2] ) )
    return mrdl_filename



def build_silo_from_orig ( filename, tolerances, destinations ):
    fdl_filename = dump_fdl( filename )
    cmd = 'java -Xms2G -Xmx2G -jar MREBuilder.jar --%s -n 3 -o %d %s' % (  
            err_func, error_offset, fdl_filename  ) 
    print cmd;
    os.system( cmd ) 

    data_files = []
    error_files = []
    for i in range( 1,4 ) :
        data_files.append( low_format % (filename, i) )
        error_files.append( 
            err_format % (filename, err_func, i, i+error_offset) )
    
    for i in range(len(tolerances)):
        error_tolerance = tolerances[i]
        destination = destinations[i]

        mrdl_filename = dump_mrdl( filename, data_files, error_files )
        print 'java -jar ARBlocksWriter.jar -f %s -e %f' % (mrdl_filename, 
                                                            error_tolerance) 
        os.system( 'java -jar ARBlocksWriter.jar -f %s -e %f' 
                    % (mrdl_filename, error_tolerance) )

        silo_filename = "%s.silo" % (filename)
        print './ar2silo -o %s %s ARBlocks' % (silo_filename, geometry_file) 
        os.system( './ar2silo -o %s %s ARBlocks' % 
                    (silo_filename, geometry_file) )

        # keep a log of block counts
        os.system( 'echo "%s" >> %s/log.txt' % (filename, destination) )
        os.system( 'ls ARBlocks/ | wc -l >> %s/log.txt' % (destination) )
        #os.system( './test_coverage.py ARBlocks >> %s/log.txt' % (destination) )
        os.system('echo "coverage test skipped" >> %s/log.txt' % (destination)) 
        
        # clean up
        print 'mv %s %s' % (silo_filename, destination) 
        os.system( 'mv %s %s' % (silo_filename, destination) )

        print 'rm -r ARBlocks'
        os.system( 'rm -r ARBlocks' )


    print 'rm -r %s.*' % (filename) 
    os.system( 'rm -r %s.*' % (filename) )



def main(argv=sys.argv):
    
    global error_offset

    if len(argv) < 3:
        print 'usage: ./build_ar_silo <data-dir> <error_offset> <error_tolerance>'
        return 1

    path = argv[1]
    error_offset = int(argv[2])
    error_tolerances = map(float, argv[3:])
    destinations = []

    path.rstrip('/')

    files = glob.glob( path + '/[!.]*' )
    files = sorted(files)
   
    for error_tolerance in error_tolerances:
        dest = dest_format % (error_offset, err_func, error_tolerance)
        destinations.append( dest )

        try:
            os.mkdir( dest, 0700 )
        except OSError:
            pass

    for file in files:
        print file + ':'
        file = os.path.abspath(file)
        build_silo_from_orig( file, error_tolerances, destinations )
    
    
    return 0

if __name__ == '__main__':
    sys.exit(main())

# Test:
# build_silo_from_orig('data/thm-20070323-j13c.bin.000780.rr')

