#! /usr/bin/python

import os
import sys
import string
import glob
import time

orig_res = [630, 200, 300]

dims_for_res = [ [630, 200, 300], 
                 [315, 100, 150],    
                 [158,  50,  75],    
                 [ 79,  25,  38],
                 [ 40,  13,  19],
                 [ 20,   7,  10],
                 [ 10,   4,   5] ]

dim_label = ['X', 'Y', 'Z']

err_func = 'avgabs'

low_format = '%s.d%d'
err_format = '%s.%s.d%d.e%d'
dest_format = 'silo/one/mr_ud/'

error_offset = 4

data_filename = "data.bin"
grid_filename = "points.grid"


geometry_file = 'thm-20070323-j13c.630x200x300.grid'
#geometry_file = 'thm-20070323-j13c.1260x400x600.grid'


fdl_contents = '''<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE FileDescriptor PUBLIC "-//SDB//DTD//ED" "fdl.dtd">
<FileDescriptor fileName= "%s"  fileType="binary" recordSize="4" byteOrder="little">
    <Bounds lower= "0"    upper= "%d"/>
    <Bounds lower= "0"    upper= "%d"/>
    <Bounds lower= "0"    upper= "%d"/>
    <Field fieldName= "temp"   fieldType= "float"/>
</FileDescriptor>
'''


def dump_fdl (filename):
    fdl_filename = filename + '.fdl'
    fdl_out = open(fdl_filename, 'w')
    fdl_out.write ( fdl_contents % 
                    (filename, orig_res[2]-1, orig_res[1]-1, orig_res[0]-1) )
    fdl_out.close()
    return fdl_filename

def dump_points (path, dims, res):
    factor = 2 ** (res + 1)
    file = open(path + grid_filename, 'w')
    print >> file, 'DATASET RECTILINEAR_GRID'
    print >> file, 'DIMENSIONS %d %d %d' % (dims[0],dims[1],dims[2])
    for i in range( 0, len(dims) ):
        print >> file, '%s_COORDINATES %d int' % (dim_label[i], dims[i])
        for n in range(0, dims[i]):
            print >> file, '%d' % (n*factor),
        print >> file, ''
    file.close()




def build_silo_for_data (data_file, dims, res, destination):
    try:
        os.mkdir( "./tmp_blocks", 0750 )
        os.mkdir( "./tmp_blocks/b1", 0750 )
    except OSError:
        pass

    os.system( "cp %s ./tmp_blocks/b1/data.bin" % (data_file) )
    dump_points("./tmp_blocks/b1/", dims, res)

    time.sleep(1)

    silo_filename = "%s.silo" % (data_file)
    cmd = './ar2silo -o %s %s tmp_blocks/' % (silo_filename, geometry_file) 
    print cmd
    os.system(cmd)

    cmd = 'mv %s %s' % (silo_filename, destination)
    print cmd
    os.system(cmd)
    
    os.system( 'rm -r tmp_blocks' )
    

def build_mr_from_orig ( filename, destination ):
    fdl_filename = dump_fdl( filename )
    cmd = 'java -Xms2G -Xmx2G -jar MREBuilder.jar --nofdl --%s -n 3 -e 3 -o 1 %s' % (err_func, fdl_filename)
    print cmd
    os.system(cmd)
   
    for d in range( 1, 4 ) :
        data_file = low_format % (filename, d)
        
        build_silo_for_data (data_file, dims_for_res[d], d, destination)

        for o in range ( 1, 4 ) :
            error_file = err_format % (filename, err_func, d, d+o)
            build_silo_for_data(error_file, dims_for_res[d+o], d+o, destination)

    cmd = 'rm -r %s.*' % (filename) 
    print cmd
    os.system( cmd )



def main(argv=sys.argv):
   
    if len(argv) < 2:
        print 'usage: ./build_mr_silo <data-dir>'
        return 1


    path = argv[1]
    destination = dest_format

    try:
        os.mkdir( destination, 0700 )
    except OSError:
        pass

    path.rstrip('/')

    files = glob.glob( path + '/[!.]*.rr' )
    files = sorted(files)

    for file in files:
        print file + ':'
        file = os.path.abspath(file)
        build_mr_from_orig( file, destination )
    
    
    return 0

if __name__ == '__main__':
    sys.exit(main())

# Test:
# build_silo_from_orig('data/thm-20070323-j13c.bin.000780.rr')

