#! /usr/bin/python

import os;
import sys;
import glob;
import re;



def read_mesh_counts(result_dir):
    mesh_table = dict()
    try:
        file = open( '%s/log.txt' % result_dir, 'r' )
        line = file.readline()
        while line != '':
            line = line.rstrip('\n')
            (_,_,name) = line.rpartition('/')
            line = file.readline() # mesh count
            meshes = line.rstrip('\n')
            line = file.readline() # coverage test.
            line = file.readline()
            
            mesh_table[name] = meshes
        file.close()
    except:
        pass
    return mesh_table

def read_expr_vars(result_dir):
    match = re.match(r'.*_r(\d)_[de](\d+\.?\d*)', result_dir)
    if match is None:
        return (0,0)
    else:
        return (match.group(1),match.group(2))

def read_sim_time(step_dir):
    match = re.match(r'.*\.(\d+)\.rr', step_dir)
    if match is None:
        return 0
    else:
        return match.group(1)

def read_render_time(step_dir):
    total = 0.0
    file = open( '%s/viewer.timings' % (step_dir), 'r' )
    for line in file.readlines():
        match = re.match(r'.*Time for first render after adding plots to this window. took (\d+\.\d+)', line) 
        if match is None:
            pass
        else:
            total += float( match.group(1) )
    return total
    

def read_io_time(step_dir):
    total = 0.0
    file = open( '%s/engine_ser.timings' % (step_dir), 'r' )
    for line in file.readlines():
        match = re.match(r'.*Reading dataset took (\d+\.\d+)', line) 
        if match is None:
            pass
        else:
            total += float( match.group(1) )
    return total

def read_engine_time(step_dir):
    total = 0.0
    file = open( '%s/viewer.timings' % (step_dir), 'r' )
    for line in file.readlines():
        match = re.match(r'.*Time for engine to execute and return data object took (\d+\.\d+)', line) 
        if match is None:
            pass
        else:
            total += float( match.group(1) )
    return total

def main(argv=sys.argv):

    if len(argv) < 2:
        print 'usage: ./time_visit.py <result_dirs>'
        return 1

    result_dirs = argv[1:]

    for dir in result_dirs:
        print dir + ':'
        dir = dir.rstrip('/')
        (_,_,name) = dir.rpartition('/')
        print name
        
        mesh_counts = read_mesh_counts(dir)
        (res_offset,error_tolerance) = read_expr_vars(dir)
        
        tab_file = open( '%s.dat' % (name), 'w' )
    
        tab_file.write('res\terror tolerance\ttime step\ttime\tmeshes\tread time\tengine time\trender time\tengine+render time\n')

        # get table of 

        time_steps = glob.glob( dir + '/[!.]*/' )
        time_steps.sort()

        for step in time_steps:
            # get all of the data.
            step = step.rstrip('/')
            (_,_,step_name) = step.rpartition('/')
            sim_time = read_sim_time(step)
            read_time = read_io_time(step)
            engine_time = read_engine_time(step)
            render_time = read_render_time(step)

            try:
                mesh_count = mesh_counts[step_name]
            except KeyError:
                mesh_count = 0

            # write to the result file.
            tab_file.write('%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%.3f\n' % 
                (res_offset, error_tolerance, step_name, sim_time, mesh_count, read_time, engine_time, render_time, float(engine_time) + float(render_time) ))
        
        tab_file.close()
        mesh_counts.clear()
    return 0

if __name__ == '__main__':
    sys.exit(main())

 
