#!/usr/bin/python

import sys
import glob
import os
import time

##################################################


low_format = '%s.d%d.silo'
err_format = '%s.maxabs.d%d.e%d.silo'

#result_dir_format = 'visit_timings/big/%s_%s'
result_dir_format = 'visit_timings/630x200x300/mr/%s_%s_.d%d.e%d'

visit_script_slice = '''import sys

OpenDatabase("%s")
AddPlot("Pseudocolor", "multivar")
AddOperator("Slice")
DrawPlots()

OpenDatabase("%s")
AddPlot("Pseudocolor", "multivar")
AddOperator("Slice")
DrawPlots()

sys.exit()
'''

visit_script_slice_diag = '''import sys

OpenDatabase("%s")
AddPlot("Pseudocolor", "multivar")
AddOperator("Slice")

t = SliceAttributes()

t.axisType = 4
t.phi = 50

SetOperatorOptions(t)

DrawPlots()

OpenDatabase("%s")
AddPlot("Pseudocolor", "multivar")
AddOperator("Slice")

t = SliceAttributes()

t.axisType = 4
t.phi = 50

SetOperatorOptions(t)

DrawPlots()

sys.exit()
'''

visit_script_volume = '''import sys

OpenDatabase("%s")
AddPlot("Volume", "multivar")
DrawPlots()

OpenDatabase("%s")
AddPlot("Volume", "multivar")
DrawPlots()

sys.exit()
'''


def dump_visit_script (data_file, error_file, visit_script_contents):
    print "dumping visit script..." 
    script = open ("visit.py", 'w')
    script.write( visit_script_contents % (data_file, error_file) )
    script.close()



resolutions = [1,2,3]
offsets = [1,2,3]

def main(argv=sys.argv):

    if len(argv) < 3:
        print 'usage: ./time_visit.py [slice|volume] <silo_dirs>'
        return 1

   
    script_contents = ''

    plot_type = argv[1]
    if plot_type == 'slice':
        script_contents = visit_script_slice
    elif plot_type == 'slice_diag':
        script_contents = visit_script_slice_diag
    elif plot_type == 'volume':
        script_contents = visit_script_volume
    else:
        print 'unknown plot type: %s' % (plot_type)
        return 1

    # set the display environment variable in case this is being run 
    # remotely
    os.putenv('DISPLAY', ':0')
    
    path = argv[2]
    set_name = 'mr_ud'

    home_dir = os.getcwd()

    cmd = 'cp -r %s /local/tmp/' % path
    print cmd
    os.system(cmd)

    
    names = []

    for name in glob.glob( '/local/tmp/' + set_name + '/[!.]*.d1.silo' ):
        (cut,_,_) = name.rpartition('.d1')
        names.append(cut)
    names = sorted(names)

    for res in resolutions:
        for off in offsets:
    
            # make a directory for the results
            result_dir = result_dir_format % (plot_type, set_name, res, res+off)
            
            try: 
                os.mkdir( result_dir, 0700 )
            except OSError:
                pass
            os.chdir( result_dir )
            
            time.sleep(1)

            for name in names:
                print name + ':'
                (fullpath, sep, filename) = name.rpartition('/')
                os.mkdir( filename, 0700 )
                os.chdir( filename )
                
                data_file = low_format % (name, res)
                error_file = err_format % (name, res, res+off)
            
                dump_visit_script( data_file, error_file, script_contents )
    
                time.sleep(1)
    
                os.system( 'umount /local/tmp' )
                
                time.sleep(1)
                
                os.system( 'mount /local/tmp' )
    
                time.sleep(1)
                os.system( 
                   'visit -cli -timing -withhold-timing-output -s visit.py' 
                )
           
                os.chdir('..')

                time.sleep(5)
            
            os.chdir( home_dir )
            
    print 'rm -rf /local/tmp/%s' % (set_name)
    os.system( 'rm -rf /local/tmp/%s' % (set_name) )

    return 0
                



if __name__ == '__main__':
    sys.exit(main())

 
