/*
 * ARBlocksWriter: comments by rdb 02/09/12
 */
package ar;

import java.io.IOException;
import org.xml.sax.SAXException;

/**
 * This just seems to be a class to test the ARDataSource.dumpBlocks method.
 * 
 * @author svohr
 */
public class ARBlocksWriter
{
   static private String input = null;
   static private String output = null;
   static private ARFloatError tolerance;

   static private void parseArguments( String[] args )
   {
      for ( int i = 0; i < args.length; i++ )
      {
         if ( args[i].equals( "-f" ) )
         {
            if ( i + 1 < args.length )
               input = args[i + 1];
            else
               System.err.println( "Missing arg for -f option." );

            i += 1;
         }
         else if ( args[i].equals( "-e" ) )
         {
            if ( i + 1 < args.length )
            {
               float e = Float.parseFloat( args[i + 1] );
               tolerance = new ARFloatError( e );
            }
            else
               System.err.println( "Missing arg for -e option." );

            i += 1;
         }
         else if ( args[i].equals( "-h" ) || args[i].equals( "-?" ) )
         {
            printUsage();
            System.exit( 0 );
         }
         else if ( input != null )
            input = args[ i];
         else
         {
            System.err.println( "Unrecognized argument: " + args[ i] );
         }
      }
      if ( input == null )
      {
         printUsage();
         System.exit( 1 );
      }
   }

   static private void printUsage()
   {
      System.err.println(
     "USAGE\n java ar.ARBlocksWriter MRinputFile [OPTIONS]\n" +
     "    MRinputFIle\n" +
     "        The MR representation used to build the tree (required).\n" +
     "    -e VALUE\n" +
     "        The error tolerance for the tree. Full tree built by default.\n" +
     "    -h -?\n" +
     "        Prints this message." );
   }

   public void processData( MRHierarchy mr, float errortolerance )
   {
      ARDataSource ar = new ARDataSource( mr );
      this.tolerance = new ARFloatError( errortolerance );
      try
      {
         // build AR representation             		  
         ar.dumpBlocks( tolerance );
      }
      catch ( Exception e )
      {
         System.err.println( e.getMessage() + " : " + e.getClass().getName() );
      }

      return;
   }

   public static void main( String[] args )
   {
      /*------------------- not needed any more -------------*
      String [] data = new String [3];
      String [] error = new String [3];
      
      // eventually these will be read in through a file.
      data[0] = "data_0.fdl";
      data[1] = "data_1.fdl";
      data[2] = "data_2.fdl";
      
      error[1] = "error_1_2.fdl";
      error[2] = "error_2_3.fdl";
      /*******************************************/

      // parse the command line arguments
      parseArguments( args );

      // Read MR from file.       
      MRHierarchy mr = new MRHierarchy();

      try
      {
         MRDLReader.readMRDLfile( mr, input );
      }
      catch ( SAXException e )
      {
      }
      catch ( IOException e )
      {
         System.err.println( e.getMessage() );
      }

      // build AR representation
      ARDataSource ar = new ARDataSource( mr );

      ar.dumpBlocks( tolerance );

      return;
   }
}
