/*
 * ARBuilder.java: comments by rdb 02/09/12
 * 
 * Edit history (recent)
 * 02/09/12 rdb: - changed code in processData to call new ARDataSource method
 *                 to write the AR xfdl file.
 *               - stored the ARDataSource that is built in an instance variable 
 *               - added a public getDataSource() method to return it. This 
 *                 makes it easy to avoid building the ARDataSource twice,
 *                 like we have been doing in MREBuilder. This is probably never
 *                 going to be used anyway since I later decided it was better to
 *                 move the writing of the fdl file to ARDataSource instead of
 *                 doing it in processData. Now there is no reason for another
 *                 class to build an ARDataSource via ARBuilder.processData.
 *               
 */

package ar;

import java.io.IOException;
import org.xml.sax.SAXException;

/**
 * Builds an AR tree representation file given an MR hierarchy of data,
 *    and an error tolerance.
 * Produces the ARDL xml file that describes the octree AR representation
 * 
 * Is this just a class to test ARDataSource and ARWriter? It seems that
 * MREBuilder does all this and more???
 * 
 * @author svohr
 */
public class ARBuilder
{
   static private String input = null;
   static private String output = null;
   static private ARFloatError tolerance;
   
   //Non-static for the above - above is used for testing
   //-------------------- instance variables ------------------------------
   private ARDataSource ards = null;
   private ARFloatError errtolerance;

   static private void parseArguments( String[] args )
   {
      for ( int i = 0; i < args.length; i++ )
      {
         if ( args[i].equals( "-o" ) )
         {
            if ( i + 1 < args.length )
               output = args[i + 1];
            else
               System.err.println( "Missing arg for -o option." );

            i += 1;
         }
         else if ( args[i].equals( "-f" ))
         {
            if ( i + 1 < args.length )
               input = args[i + 1];
            else
               System.err.println( "Missing arg for -f option." );

            i += 1;
         }
         else if ( args[i].equals( "-e" ))
         {
            if ( i + 1 < args.length )
            {
               float e = Float.parseFloat( args[i + 1] );
               tolerance = new ARFloatError( e );
            }
            else
               System.err.println( "Missing arg for -e option." );

            i += 1;
         }
         else if ( args[i].equals( "-h" ) || args[i].equals( "-?" ))
         {
            printUsage();
            System.exit( 0 );
         }
         else if ( input != null )
            input = args[ i ];
         else
         {
            System.err.println( "Unrecognized argument: " + args[ i ]);
         }
      }
      if ( input == null )
      {
         printUsage();
         System.exit( 1 );
      }
   }

   static private void printUsage()
   {
      System.err.println(
     "USAGE\n java ar.ARBuilder MRinputFile [OPTIONS]\n" +
     "    MRinputFIle\n" +
     "        The MR representation used to build the tree (required).\n" +
     "    -o OUT_FILE\n" +
     "        The output file for the ARDL information. STDOUT is default.\n" +
     "    -e VALUE\n" +
     "        The error tolerance for the tree. Full tree built by default.\n" +
     "    -h -?\n" +
     "        Prints this message." );
   }

   public void processData( MRHierarchy mr, float errortolerance )
   {
      ards = new ARDataSource( mr );
      this.errtolerance = new ARFloatError( errortolerance );
      
      //ards.writeARfdl( output );
      ards.writeARfdl( "testsend0-ar" ); // Bochichio's hack
      
      /**
      try
      {
         ARWriter out = new ARWriter();
         output = "testsend0-ar"; //rdb: looks like a Bochichio hack to test
                                  //     simulation time version, this should
                                  // be set by a parameter to the method
         if ( output != null )
            out.setOutputFile( output );

         out.writeAR( ards );
         out.close();
      }
      catch ( IOException e )
      {
         System.err.println( e.getMessage() );
      }
      /********************/

      return;
   }
   //------------- getDataSource() ---------------------------
   /**
    * Returns the ARDataSource that was built if this class is invoked by the 
    * processData method. If processData has not been called, null is returned.
    * 
    * @return 
    */
   public ARDataSource getDataSource()
   {
      return ards;
   }

   public static void main( String[] args )
   {
      /*------- hard coded tests not needed any more ----*
      String [] data = new String [3];
      String [] error = new String [3];
      
      data[0] = "data_0.fdl";
      data[1] = "data_1.fdl";
      data[2] = "data_2.fdl";
      
      error[1] = "error_1_2.fdl";
      error[2] = "error_2_3.fdl";
      /*********************/

      // parse the command line arguments
      parseArguments( args );

      // Read MR from file.
      MRHierarchy mr = new MRHierarchy();

      try
      {
         MRDLReader.readMRDLfile( mr, input );
      }
      catch ( SAXException e )
      {
      }
      catch ( IOException e )
      {
         System.err.println( e.getMessage() );
      }

      // build AR representation
      ARDataSource ar = new ARDataSource( mr );

      // write the ARDL file
      try
      {
         ARWriter out = new ARWriter();

         if ( output != null )
            out.setOutputFile( output );

         out.writeAR( ar );
         out.close();
      }
      catch ( IOException e )
      {
         System.err.println( e.getMessage() );
      }

      return;
   }
}
