/*
 * ARDLHandler.java - documentation by rdb 02/09/12
 */

package ar;

import edu.unh.sdb.datasource.DataSource;
import edu.unh.sdb.datasource.ISBounds;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Content handler for AR xml files; called by the SAX parser when
 * elements in the file are read.
 * 
 * @author svohr
 * Edited
 * 12/19/11 - rdb: DataSource is being passed to ARDataSource.setDataSource
 *                 when a DataBlock is needed; I just extracted the DB and
 *                 sent it. Why doesn't this method accept a DataSource or
 *                 DataCollection here?
*/
public class ARDLHandler implements ContentHandler 
{
    private Locator locator;
    private ARDataSource ar;
    private ARNode currentNode;
    private ARNodeBlock currentBlock;
    
    public ARDLHandler ( ARDataSource toBuild )
    {
        ar = toBuild;
        currentNode = null;
        currentBlock = null;
    }

    public void setDocumentLocator( Locator locator ) 
    {
        this.locator = locator;
    }

    public void startDocument() throws SAXException 
    {
        // nothing right now
    }

    public void endDocument() throws SAXException 
    {
        // nothing right now
    }

    public void startPrefixMapping(String prefix, String uri) 
            throws SAXException 
    {
        // nothing
    }

    public void endPrefixMapping(String prefix) throws SAXException 
    {
        // nothing
    }

    public void startElement( String uri, String localName, String qName, 
                              Attributes atts) 
                        throws SAXException 
    {
        if ( localName.equals( "ar_descriptor" ) ) 
        {
            int resolutions = Integer.parseInt(atts.getValue( "resolutions" ));
            ar.setResolutions(resolutions);            
        }
        else if ( localName.equals( "source" )) 
        {
            int res = Integer.parseInt(atts.getValue( "resolution" ));
            String path = atts.getValue( "desc" );
            DataSource ds = DataSource.create( path, path );
            ds.activate();
            //rdb edited 12/19/11 to get by compilation:
            //ar.setDataSource(ds, res);
            ar.setDataSource( ds.subblock( ds.getBounds()), res );
                       
            ar.setDataFDLNames( path, res );
        }
        else if ( localName.equals( "tree" )) 
        {
        }        
        else if ( localName.equals( "node" )) 
        {
            ARNode node = new ARNode();
            if ( currentNode == null ) 
                ar.setRoot(node);
            else
                currentNode.addChild(node);
            currentNode = node;
            
        }
        else if ( localName.equals("block")) 
        {
            int res = Integer.parseInt(atts.getValue("resolution"));
            currentBlock = new ARNodeBlock(res);
            currentNode.setBlock(currentBlock);
        }
        else if ( localName.equals("bounds")) 
        {
            String [] lCoordString = atts.getValue("lower").split(",");
            String [] uCoordString = atts.getValue("upper").split(",");
            
            int [] lCoord = new int [lCoordString.length];
            int [] uCoord = new int [uCoordString.length];
            
            for (int i=0; i<lCoord.length; i++) {
                lCoord[i] = Integer.parseInt(lCoordString[i]);
                uCoord[i] = Integer.parseInt(uCoordString[i]);
            }

            ISBounds b = new ISBounds (lCoord, uCoord);
            
            if ( currentBlock != null )
                currentBlock.setBounds(b);
            else if ( currentNode != null )
                currentNode.setBounds(b);
            else
                ar.setBounds(b);
        }
        else if ( localName.equals( "error" )) 
        {
            // create a new error
            ARError e = 
                new ARFloatError( Float.parseFloat(atts.getValue( "value" )) );
            currentBlock.setError(e);
        }
        else 
            throw new SAXException ();
    }

    public void endElement(String uri, String localName, String qName) 
            throws SAXException 
    {
        if ( localName.equals( "ar_descriptor" ) ) {
            // nothing
        }
        else if ( localName.equals( "source" )) {
            // nothing
        }
        else if ( localName.equals( "tree" )) {
            // nothing
        }
        else if ( localName.equals( "node" )) {
            currentNode = (ARNode)currentNode.getParent();
        }
        else if ( localName.equals( "block" )) {
            currentBlock = null;
        }
        else if ( localName.equals( "bounds" )) {
            // nothing
        }
        else if ( localName.equals( "error" )) {
            // nothing
        }
        else {
            throw new SAXException ();
        }
    }

    public void characters(char[] ch, int start, int length) 
            throws SAXException
    {
        // there should be no element data.
    }

    public void ignorableWhitespace(char[] ch, int start, int length) 
            throws SAXException 
    {
        // nothing
    }

    public void processingInstruction(String target, String data) 
            throws SAXException 
    {
        // nothing
    }

    public void skippedEntity(String name) throws SAXException {
        // nothing
    }

}
