/*
 * ARFloatError.java - documentation by rdb, 09/02/2012
 */

package ar;

/**
 * Represents an unspecified error that is stored as a float. 
 * 
 * 02/09/12 rdb: Added a new constructor that takes an extra string argument
 *               describing the kind of error. This string is printed on a call
 *               to typeString. Default value is "float" to remain consistent
 *               with previous code.
 * Issues: 
 *   1. Shouldn't the class encapsulate the kind of error it is,
 *      such as Max, RMSE, etc. This would require multiple ARFloatError
 *      classes, but there must be multiple instances of these anyway.
 *      Alternatively, the constructor would have a string argument
 *      that describes the kind of error.
 *      In order to do this, we'll need more information in the FDL file
 *      to specify multiple kinds of error and the names of that error.
 * 
 * @author svohr
 */
public class ARFloatError implements ARError 
{
    protected float value;
    protected String errorType = "float";
    
    //------------- constructors -------------------------------
    public ARFloatError( String errType, float f )
    {
       errorType = errType;
       value = f;
    }

    public ARFloatError( float f )
    {
       value = f;
    }

    public boolean withinTolerance( ARError e ) 
    {
        ARFloatError f = (ARFloatError) e;
        
        return ( value <= f.getValue() );
    }

    public void noError()
    {
        value = 0;
    }
    
    public float getValue() {
        return value;
    }

    public void setValue( float value ) {
        this.value = value;
    }
    
    public String valueString()
    {
        return String.valueOf(value);
    }
    
    public String typeString()
    {
        return errorType;
    }
    
}
