/*
 * ar.ARNodeBlock.java - documentation provided by rdb Feb 2012.
 */

package ar;

import edu.unh.sdb.datasource.ISBounds;

/**
 * This class seems to encapsulate the description of the data associated
 * with this node in the AR octree. However, it has no data in it and no 
 * reference to anything that has data in it. It does include information
 * about the error of the associated data compared to the original.
 * 
 * @author svohr
 * 
 */
public class ARNodeBlock 
{        
    /**
     * The resolution of the data attached to this node.
     */
    protected int resolution;
    
    /**
     * The error value associated with the resolution of the data attached 
     * to this node.
     */
    protected ARError error;
    
    /**
     * The data associated with the region.
     */
    protected ISBounds bounds;
    
    public ARNodeBlock(int res)
    {
        resolution = res;
        error = null;
        bounds = null;
    }
   
    public ARNodeBlock(int res, ARError e, ISBounds b)
    {
        resolution = res;
        error = e;
        bounds = b;
    }
    
    public ARNodeBlock(ARNodeBlock b)
    {
        resolution = b.resolution;
        error = b.error;
        bounds = b.bounds.copy();
    }
    
    public ARNodeBlock copy ()
    {
        return new ARNodeBlock (this);
    }
    
    public ARNodeBlock subBlock (ISBounds b)
    {
        if ( !bounds.contains(b) )
            return null; // bad bounds... maybe throw an exception someday
        
        ARNodeBlock block = this.copy();
        
        if ( !b.equals(bounds) ) {
            block.bounds = b.copy();
        }
        
        return block;
    }
    
    public int getResolution() {
        return resolution;
    }

    public void setResolution(int resolution) {
        this.resolution = resolution;
    }

    public ARError getError() {
        return error;
    }

    public void setError(ARError error) {
        this.error = error;
    }

    public ISBounds getBounds() {
        return bounds;
    }

    public void setBounds(ISBounds b) {
        this.bounds = b;
    }

    public boolean withinTolerance (ARError e)
    {
        return error.withinTolerance (e);
    }
    
    public void dump (String prefix) {
        System.err.println(prefix + "AST Node Block");
        System.err.println(prefix + "Data at " + resolution 
                + " with error " +  error.valueString() 
                + " (" + error.typeString() + ")");
        System.err.println(prefix + bounds.toString());
        //System.err.println(prefix + data.toString());
    }
    
}
