/*
 * MRDLHandler.java : comments by rdb 02/09/12
 * 
 * Edit history (recent)
 * 12/19/11 - rdb: DataSource is being passed setData and SetError
 *                 when a DataBlock is needed; I just extracted the DB and
 *                 sent it. Why doesn't the MRHierarchy class accept a
 *                 DataCollection here?
 */

package ar;

import edu.unh.sdb.datasource.DataSource;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Content handler for xml parser when reading xml file describing an MR data.
 * 
 * @author svohr
 * 
 */
public class MRDLHandler implements ContentHandler 
{
    private Locator locator;
    
    private MRHierarchy mr;
    
    // current values
    private int currentLevel;
    
    public MRDLHandler ( MRHierarchy mr )
    {
        this.mr = mr;
        currentLevel = 0;
    }

    public void setDocumentLocator(Locator locator) {
        this.locator = locator;
    }

    public void startDocument() throws SAXException {
        // nothing right now
    }

    public void endDocument() throws SAXException {
        // nothing right now
    }

    public void startPrefixMapping(String prefix, String uri) 
            throws SAXException 
    {
        // nothing
    }

    public void endPrefixMapping(String prefix) throws SAXException {
        // nothing
    }

    public void startElement(String uri, String localName, String qName, 
            Attributes atts) throws SAXException 
    {
        if ( localName.equals("mr_descriptor") ) {
            int level = Integer.parseInt(atts.getValue("levels"));
            mr.setLevels(level);
            
            int offset = Integer.parseInt(atts.getValue("offset"));
            mr.setErrorOffset(offset);
        }
        else if ( localName.equals("level")) {
            int level = Integer.parseInt(atts.getValue("id"));
            currentLevel = level;
        }        
        else if ( localName.equals("source")) {
            String type = atts.getValue("type");
            int res = Integer.parseInt(atts.getValue("resolution"));
            String path = atts.getValue("desc");
            
            DataSource ds = DataSource.create(path, path);
            ds.activate();
            if (type.equalsIgnoreCase("data")) {
                //rdb edited: 12/19/11 to get by compiler
                mr.setData( ds.subblock( ds.getBounds()), currentLevel );
                //rdb mr.setData(ds, currentLevel);
                mr.setDataFDLNames(path, currentLevel);
            }
            else if (type.equalsIgnoreCase("error")) {
                //rdb edited: 12/19/11 to get by compiler
                mr.setError( ds.subblock( ds.getBounds()), currentLevel );
                //rdb mr.setError(ds, currentLevel);
                mr.setErrorFDLNames(path, currentLevel);
            }
        }
        else {
            throw new SAXException ();
        }
        

    }

    public void endElement(String uri, String localName, String qName) 
            throws SAXException 
    {
        
    }

    public void characters(char[] ch, int start, int length) 
            throws SAXException
    {
        // there should be no element data.
    }

    public void ignorableWhitespace(char[] ch, int start, int length) 
            throws SAXException 
    {
        // nothing
    }

    public void processingInstruction(String target, String data) 
            throws SAXException 
    {
        // nothing
    }

    public void skippedEntity(String name) throws SAXException {
        // nothing
    }

}
