/*
 * MRHierarchy.java: comments by rdb 02/09/12
 */

package ar;

import edu.unh.sdb.datasource.DataBlock;

/**
 * Represents an MR hierarchy of data. It is more than the Granite 
 *  MRDataSource in that it also represents error data. 
 * 
 * Problem: It uses arrays of DataBlocks for the data and error; this
 *      means that everything must be in memory. That won't work for
 *      very large datasources. 
 * It should probably contain an MRDataSource object; it might be
 * acceptable for the error data to be stored as DataBlocks, since
 * it should be at a much lower resolution than the data itself, but
 * for consistency maybe it should also be an separate MRDataSource.
 * 
 * @author svohr
 * 
 * Edit history
 * 12/2011 rdb: renamed from the misspelled, MRHeirarchy.java
*/
public class MRHierarchy 
{
    // Data members to store the data and error sources.
    private DataBlock[] data;
    private DataBlock[] error;
    
    private String[]    dataFDLNames;
    private String[]    errorFDLNames;
    
    // number of levels in the hierarchy.
    private int levels;
    private int errorOffset;
    
    public MRHierarchy() 
    {
        data = null;
        error = null;
        dataFDLNames = null;
        errorFDLNames = null;
        levels = 0;
        errorOffset = 0;
    }
    
    public MRHierarchy( int levels ) 
    {
        this.levels = levels;
        errorOffset = 0;
        data = new DataBlock[ levels ];
        error = new DataBlock[ levels ];
        dataFDLNames = new String[ levels ];
        errorFDLNames = new String[ levels ];
    }

    public DataBlock[] getData() 
    {
        return data;
    }

    public void setData( DataBlock[] data ) 
    {
        this.data = data;
    }
    
    public void setData( DataBlock data, int index ) 
    {
        this.data[index] = data;
    }

    public DataBlock[] getError() 
    {
        return error;
    }

    public void setError( DataBlock[] error ) 
    {
        this.error = error;
    }
    
    public void setError( DataBlock error, int index ) 
    {
        this.error[index] = error;
    }
    
    public String[] getDataFDLNames() 
    {
        return dataFDLNames;
    }
    
    public void setDataFDLNames( String s, int index ) 
    {
        dataFDLNames[index] = s;
    }
    
    public String[] getErrorFDLNames() 
    {
        return errorFDLNames;
    }
    
    public void setErrorFDLNames( String s, int index ) 
    {
        errorFDLNames[index] = s;
    }
    
    public int getLevels() 
    {
        return levels;
    }
    
    public void setLevels( int levels ) 
    {
        this.levels = levels;
        if ( this.data == null )
             this.data = new DataBlock[ levels ];
        if ( this.error == null )
             this.error = new DataBlock[ levels ];
        if ( this.dataFDLNames == null )
             this.dataFDLNames = new String[ levels ];
        if ( this.errorFDLNames == null )
             this.errorFDLNames = new String[ levels ];
    }

    public int getErrorOffset() 
    {
        return errorOffset;
    }

    public void setErrorOffset( int errorOffset ) 
    {
        this.errorOffset = errorOffset;
    }
    
}
