/**
 * rdb: Not clear what is "multi" about this. It just calls MREBuilder once.
 * Here's what it is, I guess:
 *      A parameter to the constructor is a 2D float array. Entry 0 is
 *        the array of ALL the data in resolution 0; Entry 1 is all the data
 *        in resolution 1, etc. 
 * 
 * @author Sam Vohr
 * 
 * Edit history
 * 122011 rdb: renamed from the misspelled, MultiMRHeirarchy.java
 */

package ar;

import data.DataInfo;
import edu.unh.sdb.common.RecordDescriptor;
import edu.unh.sdb.datasource.BasicBlock;
import edu.unh.sdb.datasource.DataCollection;
import edu.unh.sdb.datasource.DataSource;
import edu.unh.sdb.datasource.FDLDescriptor;
import edu.unh.sdb.datasource.ISBounds;
import error.MREBuilder;

public class MultiMRHierarchy {
  
 
	private BasicBlock [] data;
	private BasicBlock [] grid;
	private ISBounds [] bounds;
 
 
	 /**
	  * Constructs an array of DataBlocks for the given data and grid
	  * of type float.  Stores the dimensions at each resolution
	  * where the first dimension of the fData and grid params are
	  * resolution values.
	  * @param fData
	
	  */
	 public MultiMRHierarchy( float[][] fData, DataInfo di ) 
    {
		bounds = new ISBounds[fData.length];
		data = new BasicBlock[fData.length];
		//grid = new BasicBlock[fData.length];
		int height = di.getHeight();
		int width = di.getWidth();
		int depth = di.getDepth();
		String filename = "";
		for( int i = 0; i < fData.length; i++ ) {

			int[] dim = new int[] {depth, height, width};
		           
			bounds[i] = new ISBounds(dim);
			data[i] = new BasicBlock( bounds[i], new RecordDescriptor("f") );
		  
			data[i].setFloats( fData[i] );
			filename = di.getOutputFDLPrefix();
			width /= 2;
			height /= 2;
			depth /= 2;
		}
		MRHierarchy mrh = new MRHierarchy( fData.length );
		mrh.setErrorOffset(0); //Fix me
		mrh.setData(data);
		
		for ( int i = 0; i < di.getNumRes(); i++ ) {
			mrh.setDataFDLNames( filename + i, i );
			mrh.setErrorFDLNames( filename + "-err-" + i, i );
		}
		
		
		MREBuilder mre = new MREBuilder();
		//mre.go( mrh, bounds, filename, di.getErrorTypes(), 
				//di.getByteOrder(), di.getNumRes(), di.getErrRes(), di.getErrTol() );
		mre.go( mrh, bounds, filename, di );		   
 }


 /**
  * Returns the array of data DataBlocks 
  * @return
  */
 public BasicBlock[] getBlocks() {
  return data;
 }
 
 /**
  * Sets the array of data DataBlocks
  * @param data
  */
 public void setBlocks(BasicBlock[] data) {
  this.data = data;
 }
 
 /**
  * Gets a DataBlock at the specified resolution
  * @param res
  * @return
  */
 public BasicBlock getBlock(int res) {
  return data[res];
 }
 
 /**
  * Sets the DataBlock at the specified resolution
  * @param data
  * @param res
  */
 public void setBlock(BasicBlock data, int res) {
  this.data[res] = data;
 }
 
 /**
  * Gets the entire grid DataBlock array
  * @return
  */
 public BasicBlock[] getFullGrid() {
  return grid;
 }
 
 /**
  * Sets the entire grid DataBlock array
  * @param data
  */
 public void setGrid(BasicBlock[] data) {
  this.grid = data;
 }

 /**
  * Gets a grid DataBlock at the specified resolution
  * @param res
  * @return
  */
 public BasicBlock getGridAtRes(int res) {
  return grid[res];
 }
 
 /**
  * Sets a grid DataBlock at the specified resolution
  * @param data
  * @param res
  */
 public void setGridAtRes(BasicBlock data, int res) {
  this.grid[res] = data;
 }
 
 
 
 /**
  * For testing purposes
  * @param args
  */
 public static void main(String[] args) {
  
  try{ 
   if(args.length < 2) {
    System.out.println("Usage: \nMultiMRHierarchy prefix" +
      " num_resolutions\n" + 
      " XFDL filenames are in format [prefix]-X.xfdl " +
      "where X is the number of the resolution");
    System.exit(0);
   }
   System.out.println(args[0]);
   
   int files = Integer.parseInt(args[1]);
   String[] filenames = new String[files];
   String prefix = args[0];
   float [][] data = new float[files][];
   
   //Resolution zero is done separately to set up some
   //initial variables.  Then we move into the for loop below
   //to process the rest of the resolutions.
   DataSource ds;
   FDLDescriptor fdl = FDLDescriptor.create( prefix + "-0.fdl" );
   ds = DataSource.create( "res-0", fdl );
   ds.activate();
   data[0] = ds.getFloats();
   
   int [] dim = ds.getDimensionsArray();
   int depth = dim[0];
   int height = dim[1];
   int width = dim[2];
   
   
   for(int i = 1; i < files; i++) {
    filenames[i] = prefix + "-" + i + ".fdl";
    fdl = FDLDescriptor.create( filenames[i] );
    ds = DataSource.create( "res-" + i, fdl);
    ds.activate();
    data[i] = ds.getFloats();
    System.out.println(ds.getDimensionsArray()[0] + " " 
      + ds.getDimensionsArray()[1] + " " 
      + ds.getDimensionsArray()[2]);
    
    //MREBuilder mr = new MREBuilder();
    //mr.go(ds, fdl);
    //System.out.println("Res finished");
   }
   //MultiMRHierarchy mrh = new MultiMRHierarchy(data, null, width, 
    // height, depth, prefix);
   
    
   
  }
  catch (Exception e) {
   System.out.println(e.getMessage());
   e.printStackTrace();
  }
  
 }
}
