/*
 * Octree.java
 */

package ar;

/**
 * An abstract class for building OctTrees. Can be extended to be made useful.
 *
 * @author Sam Vohr (svohr@unh.edu)
 */
public abstract class OctreeNode 
{
   
    // parents and children for the octree
    protected OctreeNode [] children;
    protected int numChildren;
    protected OctreeNode parent;
   
    
    public OctreeNode ( ) 
    {
        children = new OctreeNode [8];
        
        for ( int i=0; i<children.length; i++ ) 
            children[i] = null;
        
        numChildren = 0;
        
        parent = null;
    }
    
    public OctreeNode ( OctreeNode n ) 
    {
        this();
        
        parent = n;
        
    }
    
    public void setParent ( OctreeNode n )
    {
        parent = n;
    }
    
    public void addChild ( OctreeNode n, int px, int py, int pz ) 
    {
        this.addChild( n, ((pz * 4) + (py * 2) + px) );
    }
    
    public void addChild (OctreeNode n)
    {
        if (numChildren < 8)
            addChild (n, numChildren);
    }
    
    public void addChild ( OctreeNode n, int p )
    {
        n.setParent(this);
        if ( children[p] == null)
            numChildren += 1;
        children [p] = n;
    }
    
    
    public void removeChild ( int px, int py, int pz )
    {
        this.removeChild ( (pz * 4) + (py * 2) + px );
    }
    
    public void removeChild ( int p )
    {
        numChildren -= 1;
        children[p].setParent( null );
        children[p] = null;
    }
    
    public OctreeNode getParent () 
    {
        return parent;
    }
    
    public OctreeNode getChild ( int p )
    {
        return children[p];
    }
    
    public OctreeNode getChild ( int px, int py, int pz )
    {
        return children[ (pz * 4) + (py * 2) + px ];
    }
    
    public OctreeNode[] getChildren ()
    {
        return children;
    }
    
    public int getNumChildren ()
    {
        return numChildren;
    }
    
    public boolean isLeaf ()
    {
        for (int i=0; i<children.length; i++) {
            if ( children[i] != null)
                return false;
        }
        return true;
    }
    
    public void clearChildren ()
    {
        for (int i=0; i<children.length; i++) {
            children[i] = null;
        }
    }
    
    public abstract void dump ( String prefix );
    public abstract void dumpAll ( String prefix );
    
}


