/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package error;

import edu.unh.sdb.common.Datum;
import edu.unh.sdb.common.RecordDescriptor;
import edu.unh.sdb.datasource.BasicBlock;
import edu.unh.sdb.datasource.DataBlock;
import edu.unh.sdb.datasource.DataCollection;
import edu.unh.sdb.datasource.ISBounds;
import edu.unh.sdb.datasource.ISIterator;
import edu.unh.sdb.datasource.IndexSpaceID;


/**
 *
 * @author svohr
 */
public class BasicReconstructor implements Reconstructor {
    

    private TrilinearInterpolator interpolator = null;
    private Support support;
    private Coords coords;
    
    private DataBlock full;
    private DataBlock half011, half101, half110; 
    private DataBlock quarter001, quarter010, quarter100, one;
    
    public BasicReconstructor ( Support s, Coords c, RecordDescriptor d )
    {
        interpolator = new TrilinearInterpolator ();
        
        // Need to keep around blocks for all possible support blocks.
        // Makes everything run faster.
        int [] dims = {2,2,2};
        full = new BasicBlock (new ISBounds ( dims ), d);
        
        dims[0] = 1;
        dims[1] = 2;
        dims[2] = 2;
        half011 = new BasicBlock (new ISBounds ( dims ), d);
        
        dims[0] = 2;
        dims[1] = 1;
        dims[2] = 2;
        half101 = new BasicBlock (new ISBounds ( dims ), d);
        
        dims[0] = 2;
        dims[1] = 2;
        dims[2] = 1;
        half110 = new BasicBlock (new ISBounds ( dims ), d);
        
        dims[0] = 1;
        dims[1] = 1;
        dims[2] = 2;
        quarter001 = new BasicBlock ( new ISBounds ( dims ), d);
        
        dims[0] = 1;
        dims[1] = 2;
        dims[2] = 1;
        quarter010 = new BasicBlock ( new ISBounds ( dims ), d);
        
        dims[0] = 2;
        dims[1] = 1;
        dims[2] = 1;
        quarter100 = new BasicBlock ( new ISBounds ( dims ), d);
        
        dims[0] = 1;
        dims[1] = 1;
        dims[2] = 1;
        one = new BasicBlock (new ISBounds ( dims ),d);
        
        support = s;
        
        coords = c;
        
    }
    
    

    /**
     * Reconstructs a volume using a low resolution vesion and an interpolation 
     * method.
     * 
     * @param inData        the low-resolution data
     * @param inBase          the bounds of the low resolution in inBase coordinates
     * @param resolution    the resolution level
     * @param outBase        the bounds of the outBase to reconstruct
     * @return              the reconstructed data block
     */
    public DataBlock reconstruct( DataCollection inData, ISBounds inBase, 
                                  int resolution, 
                                  DataBlock outData, ISBounds outBase    ) 
    {
      
        Datum reconstructedDatum = outData.createDatum();
        
        // for each point in the new block...
        ISIterator outIter = new ISIterator( outData.getBounds() );
        int [] index = new int [3];
        int [] baseOffset = inBase.getLowerArray();
        
        // Need to compute an offset between the in and out base bounds.
        // These bounds should both be for points in the original data
        // set (even numbered lower/upper coords).
        
        int [] inOutOffset = new int [3];
        int [] inLow = inBase.getLowerArray();
        int [] outLow = outBase.getLowerArray();
        for ( int i=0 ; i< inOutOffset.length ; i++ ) {
            inOutOffset[i] = outLow[i] - inLow[i];
            inOutOffset[i] /= 2; // move to original coords.
        }
        
        
        
        // Store previous supportBounds
        ISBounds prevSupport = null;
        
        // Data for the point to be reconstructed.
        DataBlock supportBlock = null;
        ISBounds baseSupportBounds = null;

        for ( outIter.init(); outIter.valid(); outIter.next() ) {
                    
            // local coords of the point to reconstuct
            outIter.getCoords(index);
            
            // Move by the in/out offset...
            for ( int i=0 ; i< index.length ; i++ ) {
                index[i] += inOutOffset[i];
            }

            // for each point in the out set, find the points that
            // supportBounds it.
            ISBounds supportBounds = 
                    support.supportForOrigPoint(resolution, index);

            // Reconcile the support with the data available.
            // If there is no intersection, use the upper bound of the
            // data set.
            if ( supportBounds.intersects(inData.getBounds()) ) {
                supportBounds.intersectWith(inData.getBounds());
            }
            else {
                int [] supportLow = supportBounds.getLowerArray();
                int [] dataUp = inData.getBounds().getUpperArray();
                
                for ( int i=0; i < supportLow.length; i++ ) {
                    if ( supportLow[i] > dataUp[i] ) 
                        supportLow[i] = dataUp[i];
                }
                
                supportBounds.setLower(supportLow);
                supportBounds.intersectWith(inData.getBounds());
            }

            assert supportBounds != null;

            // See if the support is the same as the last iteration...
            // Maybe save a subblock operation...
            if ( prevSupport == null || !supportBounds.equals(prevSupport) ) {
                
                supportBlock = this.getDataBlock(inData, supportBounds);

                // move bounds to world inBase coords
                baseSupportBounds = coords.lowResToBase(resolution, supportBounds);

                baseSupportBounds.move( inBase.getLower() );

            }
            // else, support is the same as the last time through...
            // don't need to grab it again.
            
            
            // move point to world inBase coords
            index = coords.lowResToBase(0, index);
            for (int l=0; l<index.length; l++) {
                index[l] += baseOffset[l];
            }


//          System.err.println ("inBase= " + supportBounds.toString());

            // interpolate the values for all attributes.
            reconstructedDatum = this.interpolator.interpolate 
                    (supportBlock, baseSupportBounds, index, reconstructedDatum);

            //outData.setDatum(outIter, reconstructedDatum);
            this.setDatum(outData, outIter, reconstructedDatum);

            prevSupport = supportBounds;
        }
        
        return outData;
    }
    
    private DataBlock getDataBlock (DataCollection in, ISBounds b) {
        int [] d = b.getDimensionsArray();
        
        assert d.length == 3;
        
        int volume = d[0] * d[1] * d[2];
        
        if ( volume == 8 ) {
            in.subblock(full,b);
            return full;
        }
        else if ( volume == 4 ) {
            if ( d[0] == 1 ) {
                in.subblock(half011,b);
                return half011;
            }
            else if ( d[1] == 1 ) {
                in.subblock(half101,b);
                return half101;
            }
            else { // if ( d[2] == 1 ) {
                in.subblock(half110,b);
                return half110;
            }
        }
        else if ( volume == 2 ) {
            if ( d[2] == 2 ) {
                in.subblock(quarter001,b);
                return quarter001;
            }
            else if ( d[1] == 2 ) {
                in.subblock(quarter010,b);
                return quarter010;
            }
            else { // if ( d[0] == 2 ) {
                in.subblock(quarter100,b);
                return quarter100;
            }
        }
        else if ( volume == 1 ) {
            in.subblock(one,b);
            return one;
        }
        return in.subblock(b);
    }
    
    private void setDatum (DataBlock b, IndexSpaceID i, Datum d) {
        b.setDatum(i, d);
    }
    

    public static void main ( String[] args ) 
    {
        
        int [] dims = {3,2,2};
        ISBounds bounds = new ISBounds (dims);
        RecordDescriptor rd = new RecordDescriptor ('f', 1);
        DataBlock block = new BasicBlock (bounds, rd);
        
        ISIterator values = new ISIterator (bounds);
        
        for ( float v=0 ; values.valid(); values.next(), v += 2.0 ) {
            if ( v < 8.0 )
                block.setFloat(values, 0, 0);
            else 
                block.setFloat(values, 0, v - 8);
        }
        
        BasicReconstructor test = 
                new BasicReconstructor( new WaveletSupport (), 
                                        new WaveletCoords (), 
                                        new RecordDescriptor ('f',1) );
        

        // Base coords for 
        int [] baseLow = {0,0,0};
        int [] baseUp = {10,6,6};
        ISBounds base = new ISBounds (baseLow, baseUp);

        
        int [] regionDims = {4,4,4};
        ISBounds region = new ISBounds (regionDims);
        DataBlock regionBlock = new BasicBlock (region, rd);
        int [] regionBaseDims = {7,7,7};
        ISBounds regionBase = new ISBounds ( regionBaseDims );
        int [] off = {4,0,0};
        //regionBase.move( off );
        

        DataBlock result = test.reconstruct(block, base, 1, regionBlock, regionBase);
        
        ISIterator it = new ISIterator (bounds);
        Datum d = block.createDatum();
        
        for ( it.init(); it.valid(); it.next() ) {
            block.datum(d, it); 
            System.err.println( d.getFloat(0) );
        }
        
        System.err.println ("-----------------------------");
        
        it = new ISIterator (region);
        
        for ( it.init(); it.valid(); it.next() ) {
            result.datum(d, it); 
            System.err.println( d.getFloat(0) );
        }
        
        //System.out.println(result.toString());
        
        return;
    }
    
}
