/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package error;

import edu.unh.sdb.common.Datum;
import edu.unh.sdb.common.RecordDescriptor;
import edu.unh.sdb.datasource.BasicBlock;
import edu.unh.sdb.datasource.DataBlock;
import edu.unh.sdb.datasource.DataCollection;
import edu.unh.sdb.datasource.FDLDescriptor;
import edu.unh.sdb.datasource.ISBounds;
import edu.unh.sdb.datasource.ISBoundsIterator;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.ByteOrder;


/**
 *
 * @author svohr
 */
public class ErrorWriter extends DataSetWriter {
    
    private ErrorFunction func;
    private int dataResolution, errorResolution;
    private int maxSlices;
    
    // Scratch blocks for error calculations
    private int [] cellDims;
    private ISBounds cellBounds;
    private DataBlock actBlock, recBlock;
    
    private Datum errDatum;
    
    public ErrorWriter (    ErrorFunction f, 
                            int d, int e, int lowest,
                            FDLDescriptor orig )
            throws FileNotFoundException
    {
        func = f;
        
        dataResolution = d;
        errorResolution = e;
        maxSlices = (int) (Math.pow(2, lowest) + 1);

        // Build a name for the new file.
        String name = new String (orig.getDataFileURL());
        
//        if ( name.matches("\\.rr$") ) {
//            name = name.substring( 0, name.lastIndexOf(".rr") );
//        }
        
        this.filename = name + "." + f.name() + ".d" + d + ".e" + e;
       
        // Set up scratch data blocks for error calculations
        cellDims = new int [3];
        
        int dim = (int) (Math.pow(2, e) + 1);
        for (int i=0; i<cellDims.length; i++)
            cellDims[i] = dim;
        cellBounds = new ISBounds (cellDims);
        
        actBlock = new BasicBlock (cellBounds, orig.getRecordDescriptor());
        recBlock = new BasicBlock (cellBounds, orig.getRecordDescriptor());
        
        RecordDescriptor rd = f.getRecordDescriptor();
        
        errDatum = Datum.create( rd );
        
        // Set up the FDL for the error set. Keep some things from the
        // original (like endianess). Most will change.
        this.fd = new FDLDescriptor (   rd, 
                                        errorSetSize(orig.getBounds()), 
                                        orig.getAxisOrdering() );
        
        //this.fd = (FDLDescriptor)orig.clone(); 
        //fd.setBounds(errorSetSize(orig.getBounds()));
        fd.setByteOrder( orig.getByteOrder() );
        fd.setDataFileType("binary");
        fd.setRecordSize( rd.byteSize() );
        
        String [] fnames = rd.getFieldNames();
        String [] ftypes = rd.getFieldTypes();
        for ( int i=0; i < fnames.length; i++ ) {
            fd.addFieldName(fnames[i]);
            fd.addFieldType(ftypes[i]);
        }
        



        // set up everthing else.
        this.openForWriting();
    }


    /**
     * Creates an ErrorWriter with a filePrefix when data is being
     * passed in through memory as opposed to via a FDLDescriptor
     * @param f
     * @param d
     * @param e
     * @param lowestResolution
     * @param filePrefix
     */
    public ErrorWriter(ErrorFunction f, int d, int e, int lowest,
			String filePrefix, ISBounds bounds, String byteOrder) 
    		throws FileNotFoundException
    {
    	
    	func = f;
        
        dataResolution = d;
        errorResolution = e;
        maxSlices = (int) (Math.pow(2, lowest) + 1);

        String name = filePrefix;
        
        this.filename = name + "." + f.name() + ".d" + d + ".e" + e;
       
        // Set up scratch data blocks for error calculations
        cellDims = new int [3];
        
        int dim = (int) (Math.pow(2, e) + 1);
        for (int i=0; i<cellDims.length; i++)
            cellDims[i] = dim;
        cellBounds = new ISBounds (cellDims);
        
        //DB: These are hard-coded as floats that are coming from the
        //simulator.  In the future, it might be a good idea to 
        //Pass in the RecordDescriptor type into this method.
        actBlock = new BasicBlock (cellBounds, new RecordDescriptor("f"));
        recBlock = new BasicBlock (cellBounds, new RecordDescriptor("f"));
        
        RecordDescriptor rd = f.getRecordDescriptor();
        
        errDatum = Datum.create( rd );
        
        // Set up the FDL for the error set. Keep some things from the
        // original (like endianess). Most will change.
        this.fd = new FDLDescriptor ();
        fd.setBounds(bounds);
        
        if ( byteOrder.equalsIgnoreCase( "big" ) )
        	fd.setByteOrder( ByteOrder.BIG_ENDIAN );
    	else
        	fd.setByteOrder( ByteOrder.LITTLE_ENDIAN );	
        
        fd.setDataFileType("binary");
        fd.setRecordSize( rd.byteSize() );
        
        String[] fnames = rd.getFieldNames();
        String[] ftypes = rd.getFieldTypes();
        for ( int i=0; i < fnames.length; i++ ) {
            fd.addFieldName( fnames[i] );
            fd.addFieldType( ftypes[i] );
        }
        



        // set up everthing else.
        this.openForWriting();
	}



	public void write ( DataCollection actual, DataCollection recon ) 
            throws IOException
    {
        // blocks need to be the same size

        assert actual.getBounds().equals(recon.getBounds());

        
        int [] gaps = {-1, -1, -1};
        ISBoundsIterator cellIter = 
                new ISBoundsIterator (actual.getBounds(), cellDims, gaps);
        
        int cellsToWrite = 
                this.numberOfCellsToWrite (actual.getDimensionsArray());
        
        for (   int i=0; 
                i < cellsToWrite && cellIter.valid(); 
                i++, cellIter.next() ) 
        {
            if ( actBlock.getBounds().equalShape(cellIter) ) {
                actual.subblock(actBlock, cellIter);
                recon.subblock(recBlock, cellIter);
            } else {
                actBlock = actual.subblock(cellIter);
                recBlock = recon.subblock(cellIter);
            }
            
            
            func.calculate(actBlock, recBlock, errDatum);
            
            this.write(errDatum);
        }
        
    }

    public void write(DataBlock actual, ISBounds errorWindow, DataBlock recon) 
            throws IOException
    {
            
        // blocks need to be the same size

        assert errorWindow.equalShape(recon.getBounds());

        
        int [] gaps = {-1, -1, -1};
        ISBoundsIterator actIter = 
                new ISBoundsIterator (errorWindow, cellDims, gaps);
        ISBoundsIterator recIter = 
                new ISBoundsIterator (recon.getBounds(), cellDims, gaps);
        
        int cellsToWrite = 
                this.numberOfCellsToWrite (errorWindow.getDimensionsArray());
        
        for (   int i=0; 
                i < cellsToWrite && actIter.valid() && recIter.valid(); 
                i++, actIter.next(), recIter.next() ) 
        {
        	DataBlock actDest = actBlock;
        	DataBlock recDest = recBlock;
            
        	if(!actIter.isFullShape()) {
            	actDest = new BasicBlock(actIter.copyBounds(), actBlock.getRecordDescriptor());
            	recDest = new BasicBlock(recIter.copyBounds(), recBlock.getRecordDescriptor());
            }
            
        	actual.subblock(actDest, actIter);
        	recon.subblock(recDest, recIter);
        
            func.calculate(actDest, recDest, errDatum);
            
            this.write(errDatum);
        }
        
    
    }
    
    private ISBounds errorSetSize ( ISBounds orig )
    {
        int [] dims = orig.copyUpperArray();
        
        for (int i=0; i<dims.length; i++) {
            dims[i] = (dims[i] / (int) (Math.pow(2, errorResolution)) + 1);
        }
        
        return new ISBounds (dims);
    }


    private int numberOfCellsToWrite ( int [] size ) 
    {        
        int dist = (int)Math.pow(2, errorResolution);
        
        int zdim = maxSlices / dist;
        int ydim = size[1] / dist;
        int xdim = size[2] / dist;
        
        if ( size[2] % dist != 0 ) 
            xdim += 1;
        if ( size[1] % dist != 0 ) 
            ydim += 1;

        return xdim * ydim * zdim;
    }
}
