/*
 * HaarWavelet.java
 *
 * Created on May 30, 2009, 10:15 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package error;


import edu.unh.sdb.common.RecordSpec;
import edu.unh.sdb.datasource.DataBlock;
import edu.unh.sdb.datasource.DataCollection;
import edu.unh.sdb.datasource.ISBounds;
import edu.unh.sdb.datasource.ISBoundsIterator;
import edu.unh.sdb.datasource.ISIterator;

/**
 *
 * @author rdb
 */
public class HaarWavelet extends Wavelet
{
    //------------------ class variables --------------------------
    //------------------ instance  variables ----------------------
    
    private RecordSpec [] attrs;

    //------------------ constructors -----------------------------
    /** Creates a new instance of HaarWavelet */
    public HaarWavelet( int dim, int res )
    {
        super( dim );
        support = (int) Math.pow(2, res);
      
      
        float coeff = (float) (Math.sqrt(2.)) / support;
      
        sumCoeff = new float[ support ];
        for ( int i=0; i<support; i++ ) {
            sumCoeff[i] = coeff;
        }
      
      
        // should never be used...
        detCoeff = new float[ support ];
        for ( int i=0; i<support; i++ ) {
            detCoeff[i] = coeff;
        }
        
        attrs = null;
      
        computeFactors();      
    }
   
      //-------- doSummary( DataCollection, ISBounds, DataBlock ) ------------
   /**
    * apply one step of wavelet to the portion of the input DataCollection 
    * specified by the bounds and return the result in the DataBlock that is 
    * passed in. If the bnds is smaller than the DataBlock, only return that
    * portion of the bounds in the DataBlock; set the remainder of the 
    * DataBlock to 0.
    */
    public void doSummary( DataCollection inData, ISBounds bnds, 
                          DataBlock outBlock )
    {
        // need output bounds to be 1/2 size of the input in all dimensions
        // first get the sizes of the input
        ISBounds outBounds = outBlock.getBounds();

        // waveBasis must be large enough in each dimension to hold the points
        //  needed for the particular kind of wavelet. 2 for Haar, 4 for D4, etc.
        // Since we need to step by 2 all the time, the gaps array must be 
        //  negative number that is 2 - support.
        int[]    waveBasis = new int[ inData.dim() ];
        int[]    gaps      = new int[ inData.dim() ];
        for ( int i = 0; i < waveBasis.length; i++ )
        {
            waveBasis[ i ] = support;
            gaps[ i ] = 0;
        }

        ISBounds waveBounds = new ISBounds( waveBasis );

        // And pre-allocated a DataBlock to take the nxnx ... x n subblock
        DataBlock waveBlock = inData.createDataBlock( waveBounds );

        // We need a block iterator that will go through the data space, but it
        //   will always step by 2 in each dimension. Hence, we need a negative
        //   gaps parameter
        ISBoundsIterator inIter = new ISBoundsIterator( bnds, waveBasis, gaps );

        // We also need an iterator over the output space, but this is a simple
        // iterator that just identifies one point at a time (not a block).
        ISIterator outIter = new ISIterator( outBounds );

        // pre-allocate an array to fill with the data
        float[] vals = new float[ waveBlock.volume() ];

        // System.out.println( "blocksize: " + vals.length );
        
        // Make a bunch of of RecordSpecs to get all of the attributes out
        // of the data block.
        if ( attrs == null ) {
            attrs = new RecordSpec 
                        [waveBlock.getRecordDescriptor().getNumAttributes()];
            for ( int i=0; i< attrs.length; i++ ) {
                attrs[i] = new RecordSpec (new int[] {i});
            }
        }

        for ( ; inIter.valid(); inIter.next() )
        {
            if ( inIter.isFullShape() ) {
                for ( int i=0; i < attrs.length; i++ ) {
                    inData.subblock( waveBlock, inIter, attrs[i] );  // get the next block
                    waveBlock.getFloats( vals );

                    // now compute the wavelet transform for the block
                    float value = 0.0f;
                    for ( int j = 0; j < vals.length; j++ ) {
                        assert (vals[j] == vals[j]);
                        value += sumFactors[ j ] * vals[ j ];
                    }
                    
                    outBlock.setFloat( outIter, i, value ); // store result
                }
                outIter.next();
            }
        }

    }
}
