/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package error;

import edu.unh.sdb.common.Datum;
import edu.unh.sdb.common.FloatDatum;
import edu.unh.sdb.common.RecordDescriptor;
import edu.unh.sdb.datasource.DataCollection;
import edu.unh.sdb.datasource.ISIterator;

/**
 *
 * @author svohr
 */
public class SignalToNoiseErrorFunction implements ErrorFunction {

    private String funcName;
    private RecordDescriptor rd;

    public SignalToNoiseErrorFunction ()
    {
        funcName = "snr";
        rd = new RecordDescriptor ('f', 1);
    }
    
    public Datum calculate( DataCollection actual, 
                            DataCollection reconstructed ) 
    {
        Datum result = new FloatDatum (1);
        
        return this.calculate(actual, reconstructed, result);
    }
    
    private float rms ( DataCollection d )
    {
        float [] values = d.getFloats(0);

        float sum = 0.f;
        float count = values.length;

        for ( int i=0; i< values.length; i++ ) {
            sum += Math.pow( values[i], 2 );
        }

        return (float) Math.sqrt( sum / count );
    }

    public Datum calculate( DataCollection actual, 
                            DataCollection reconstructed, Datum result ) 
    {
        float [] actualValues = actual.getFloats(0);
        float [] reconstructedValues = reconstructed.getFloats(0);
        
        float noiseSum = 0.f;
        float count = actualValues.length;

        for (int i=0; i< actualValues.length; i++ ) {
            noiseSum 
                += Math.pow ( actualValues[i] - reconstructedValues[i], 2.);
        }

        float ampSignal = rms(actual);
        float ampNoise = (float)Math.sqrt(noiseSum / count);

        if (ampNoise < 0.001) {
            ampNoise  = 0.001f;
        }
        
        result.setFloat(0, 20.f * (float) Math.log10(ampSignal / ampNoise) );
        
        return result;
    }

    public String name() 
    {
        return this.funcName;
    }

    public RecordDescriptor getRecordDescriptor() 
    {
        return this.rd;
    }

}
