/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package error;

import edu.unh.sdb.common.Datum;
import edu.unh.sdb.common.RecordDescriptor;
import edu.unh.sdb.datasource.BasicBlock;
import edu.unh.sdb.datasource.DataBlock;
import edu.unh.sdb.datasource.ISBounds;
import edu.unh.sdb.datasource.ISIterator;

/**
 *
 * @author svohr
 */
public class TrilinearInterpolator {
    
    private Mapper indexMapper;
    private float [] buffer;
    
    private float [] pointCoords;
    private float [] lowerCoords;
    private float [] upperCoords;
    
    private  int [] index;
    
    public TrilinearInterpolator ()
    {
        indexMapper = new ISMapper ();
        buffer = new float [8];
        
        pointCoords = new float [3];
        lowerCoords = new float [3];
        upperCoords = new float [3];
        
        index = new int [] {0, 0, 0};
    }
    
    public TrilinearInterpolator (Mapper m)
    {
        indexMapper = m;
        buffer = new float [8];
        
        pointCoords = new float [3];
        lowerCoords = new float [3];
        upperCoords = new float [3];
        
        index = new int [] {0, 0, 0};
    }

    
    private float interpolate1d (float v1, float p1, 
                                 float v2, float p2, float p3)
    {
        // check if the points are the same.
        if (p2 == p1) {
            return v1; 
        }
        
        /*
         * p1 -----------p3---------------- p2
         */
        float result =  (v1 * ( (p2 - p3) / (p2 - p1) ) ) +
                        (v2 * ( (p3 - p1) / (p2 - p1) ) );
        // System.err.printf ("%f (%f) --- %f (%f) --- %f (%f)\n",
        //                    v1, p1, result, p3, v2, p2);
        return result;

    }
    
    
    private float interpolate ( float [] corners, float [] pCoords, 
                                float [] lCoords, float [] uCoords )
    {
        //System.err.println (corners[0] + " " + corners[1] + " \t" + corners[4] + " " + corners[5]);
        //System.err.println (corners[2] + " " + corners[3] + " \t" + corners[6] + " " + corners[7]);
        //System.err.println ("(" + pCoords[0] + "," + pCoords[1] + "," + pCoords[2] + ")");
        //System.err.println ("(" + lCoords[0] + "," + lCoords[1] + "," + lCoords[2] + ")");
        //System.err.println ("(" + uCoords[0] + "," + uCoords[1] + "," + uCoords[2] + ")");
        // values for interpolation
        float v00x, v01x, v10x, v11x;
        float v0yx, v1yx, vzyx;

        // do all the interpolation
        v00x = interpolate1d (  corners[0], lCoords[2], 
                                corners[1], uCoords[2], pCoords[2] );
        v01x = interpolate1d (  corners[2], lCoords[2], 
                                corners[3], uCoords[2], pCoords[2] );
        v10x = interpolate1d (  corners[4], lCoords[2], 
                                corners[5], uCoords[2], pCoords[2] );
        v11x = interpolate1d (  corners[6], lCoords[2], 
                                corners[7], uCoords[2], pCoords[2] );

        v0yx = interpolate1d (  v00x, lCoords[1],
                                v01x, uCoords[1], pCoords[1] );
        v1yx = interpolate1d (  v10x, lCoords[1],
                                v11x, uCoords[1], pCoords[1] );

        vzyx = interpolate1d (  v0yx, lCoords[0],
                                v1yx, uCoords[0], pCoords[0] );
        
        //System.err.println (vzyx);
        
        return vzyx;
    }
    

    
    
    
    public Datum interpolate (  DataBlock values, ISBounds base, 
                                int [] point, Datum res ) 
    {
        //----------------------------------------------------
//        System.err.println ("------------------------------");
//        System.err.println ("interpolate");
//        System.err.println ("bounds = " + values.getBounds().toString());
//        System.err.println ("values: ");
//        ISIterator it = new ISIterator (values.getBounds());
//        Datum d = values.createDatum();
//        
//        for ( it.init(); it.valid(); it.next() ) {
//            values.datum(d, it); 
//            System.err.println( d.getFloat(0) );
//        }
//        System.err.println ("base = " + base.toString());
//        System.err.println ("point = (" + point[0] + "," + point[1] + "," + point[2] + ")");
//        System.err.println ("------------------------------");
        //----------------------------------------------------
        

        
        // interpolate
        float [] pCoords = indexMapper.map (point, pointCoords);

        base.copyLowerArray(index);
        float [] lCoords = indexMapper.map (index, lowerCoords);
        base.copyUpperArray(index);
        float [] uCoords = indexMapper.map (index, upperCoords);
        
        
//        System.err.println ("--lpoint = (" + lCoords[0] + "," + lCoords[1] + "," + lCoords[2] + ")");
//        System.err.println ("--upoint = (" + uCoords[0] + "," + uCoords[1] + "," + uCoords[2] + ")");
        

        for ( int attr=0; attr < values.getNumAttributes(); attr++ ) {
            // this should be 8 values; one for each corner
            float [] vals = values.getFloats(attr);
            float [] corners;

            // check if we have all 8 values.
            if ( vals.length == 8 ) {
                
                corners = vals;
                res.setFloat(attr, 
                        interpolate(corners,pCoords,lCoords,uCoords) );
            }
            else if ( vals.length == 4 ) {
                // must fill in missing values.
                corners = buffer;

                // number of values is 4.
                int [] dims = values.getDimensionsArray();

                if ( dims[0] == 1 ) {
                    for (int i=0; i<corners.length; i++)
                        corners[i] = vals[i%4];
                }
                else if ( dims[1] == 1 ) {
                    for (int i=0; i<corners.length; i++) 
                        corners[i] = vals[2*(i/4) + (i%2)];
                }
                else { // if ( dims[2] == 1 )
                    for (int i=0; i<corners.length; i++) 
                        corners[i] = vals[i/2];
                }
                
                res.setFloat(attr, 
                        interpolate(corners,pCoords,lCoords,uCoords) );
            }
            else if ( vals.length == 2 ) {
                // must fill in missing values.
                corners = buffer;
                
                int [] dims = values.getDimensionsArray();
                
                if ( dims[0] == 1 && dims[1] == 1 ) {
                    for (int i=0; i < corners.length; i++) 
                        corners[i] = vals[i%2];
                }
                else if ( dims[0] == 1 && dims[2] == 1 ) {
                    for (int i=0; i < corners.length; i++)
                        corners[i] = vals[(i/2) % 2];
                }
                else { // ( dims[1] == 1 && dims[2] == 1 )
                    for (int i=0; i < corners.length; i++)
                        corners[i] = vals[i/4];
                }
                
                float val = interpolate(corners,pCoords,lCoords,uCoords);
                
                float max = corners[0];
                for ( int x=1; x < corners.length; x++ ) {
                    if (corners[x] > max)
                        max = corners[x];
                }
                try {
                    assert val <= max + 1;
                }
                catch (AssertionError a) {
                    System.err.println ("result=" + val);
                    for ( int x=1; x < corners.length; x++ ) {
                        System.err.print(corners[x] + " ");
                    }
                    System.err.println();
                    System.err.println("base=" + base.toString());
                    System.err.println("point=" + point[0] +"," + point[1] +","+ point[2]);
                    System.exit(1);
                }
                
                
                res.setFloat(attr, val );
                
            }
            else { // if ( vals.length == 1 ) 
                // Only 1 value. No interpolation necessary.
                res.setFloat(attr, vals[0]);
            }

        }
        
        return res;
    }
    
    public static void main( String[] args )
    {
        // Test cases
        int [] dims = {2,2,2};
        ISBounds bounds = new ISBounds (dims);
        RecordDescriptor rd = new RecordDescriptor ('f', 1);
        DataBlock block = new BasicBlock (bounds, rd);
        
        ISIterator values = new ISIterator (bounds);
        
        for ( float v=0 ; values.valid(); values.next(), v += 1.0 ) {
            block.setFloat(values, 0, v);
        }
        
        int [] baseDims = {10,20,30};
        ISBounds base = new ISBounds (baseDims);
        int [] point = {3,17,19};
        TrilinearInterpolator e = new TrilinearInterpolator ();
        
        //float result = e.interpolate(block, base, point);
        
        //System.out.println(result);
        
        return;
    }
    
}
