/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package error;

import edu.unh.sdb.common.Datum;
import edu.unh.sdb.common.RecordDescriptor;
import edu.unh.sdb.datasource.BasicBlock;
import edu.unh.sdb.datasource.DataBlock;
import edu.unh.sdb.datasource.DataCollection;
import edu.unh.sdb.datasource.ISBounds;
import edu.unh.sdb.datasource.ISIterator;

/**
 *
 * @author svohr
 */
public class UniformDecimation implements Reducer
{
    
    private int interval;
    private int resolution;
    private int [] skip;
    
    private Coords coords;
    
    public UniformDecimation () 
    {
        // default is first resolution.
        this.interval = 2;
        
        // Build the step array.
        skip = new int [3];
        for (int i=0; i<skip.length; i++)
            skip[i] = interval;
    }
    
    public UniformDecimation ( int r ) 
    {
        this.resolution = r;
        this.interval = (int)Math.pow(2, resolution);
        
        // Build the step array.
        skip = new int [3];
        for (int i=0; i<skip.length; i++)
            skip[i] = interval;
        
        coords = new UniformCoords ();
    }
    
    public DataBlock reduce ( DataCollection inData ) 
    {
        // Make a DataBlock to return the summary data

        ISBounds inBounds = inData.getBounds();
        
        return this.reduce(inData, inBounds);

    }
        
    public DataBlock reduce ( DataCollection inData, DataBlock outBlock )
    {
        return reduce (inData, inData.getBounds(), outBlock);
    }
    
    public DataBlock reduce ( DataCollection inData, ISBounds inBounds )
    {
        // need output bounds to be 1/2 size of the input in all dimensions
        // first get the sizes of the input
        int[]  outSizes = inBounds.getDimensionsArray();

        // now halve them -- It's not really half...
        for ( int i = 0; i < inData.dim(); i++ )
            outSizes[i] = (outSizes[i] + (interval-1)) / interval;
        // now make a bounds for the output
        ISBounds outBounds = new ISBounds( outSizes );

        // get an output block with the right bounds and just one attribute
        DataBlock outBlock  = inData.createDataBlock( outBounds );

        this.reduce ( inData, inBounds, outBlock );

        return outBlock;
    }


    public DataBlock reduce ( DataCollection inData, ISBounds inBounds, 
                                DataBlock outBlock )
    {
        // need output bounds to be 1/2 size of the input in all dimensions
        // first get the sizes of the input
        ISBounds outBounds = outBlock.getBounds();

        // We need a block iterator that will go through the data space, but it
        //   will always step by 2 in each dimension. Hence, we need a negative
        //   gaps parameter
        ISIterator inIter = new ISIterator( inBounds, skip );

        // We also need an iterator over the output space, but this is a simple
        // iterator that just identifies one point at a time (not a block).
        ISIterator outIter = new ISIterator( outBounds );

        Datum dat = inData.createDatum();
        
        // Get the sample points from the data set. 
        for ( ; inIter.valid(); inIter.next(), outIter.next() )
        {
            inData.datum(dat, inIter);
            outBlock.setDatum(outIter, dat);
        }
        
        return outBlock;
    }
    
    public ISBounds resultSize ( ISBounds original ) 
    {
        int [] dims = new int [3];
        
        original.getDimensions(dims);

        for (int i=0; i < dims.length ; i++) 
            dims[i] = ( (dims[i] + interval - 1) / interval);

        return new ISBounds (dims);
    }
    
    public ISBounds resultBounds ( ISBounds original )
    {

        // Get the original offset...
        int [] lower = original.copyLowerArray();

        // Get the size of the new data set...
        ISBounds result = this.resultSize(original);

        
        // Move the result to the offset
        result = coords.lowResToBase(resolution, result);
        lower = coords.lowResToBase(0, lower);

        result.move(lower);

        return result;
    }
    
    
    public static void main (String [] args)
    {
        int [] dims = {10,10,10};
        ISBounds bounds = new ISBounds (dims);
        RecordDescriptor rd = new RecordDescriptor ('f', 1);
        DataBlock block = new BasicBlock (bounds, rd);
        
        ISIterator values = new ISIterator (bounds);
        
        for ( float v=0 ; values.valid(); values.next(), v += 1.0 ) {
            block.setFloat(values, 0, v);
        }
        
        UniformDecimation ud = new UniformDecimation (1);
        
        int [] resultDims = {5,5,5};
        ISBounds resultBounds = new ISBounds (resultDims);
        DataBlock result = new BasicBlock (resultBounds, rd);
        
        ud.reduce(block, result);
        
        ISIterator it = new ISIterator (resultBounds);
        Datum d = result.createDatum();
        

        for ( it.init(); it.valid(); it.next() ) {
            result.datum(d, it); 
            System.err.println( d.getFloat(0) );
        }
    }

    public ISBounds resultInData(ISBounds original) {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
