/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package error;

import edu.unh.sdb.common.Datum;
import edu.unh.sdb.common.RecordDescriptor;
import edu.unh.sdb.datasource.BasicBlock;
import edu.unh.sdb.datasource.DataBlock;
import edu.unh.sdb.datasource.DataCollection;
import edu.unh.sdb.datasource.ISBounds;
import edu.unh.sdb.datasource.ISIterator;

/**
 *
 * @author svohr
 */
public class WaveletReducer implements Reducer {
    
    private int [] offset;
    private int resolution;
    private int support;
    
    private HaarWavelet wavelet;
    
    private Coords coords;
    
    public WaveletReducer ( int r ) 
    {
        resolution = r;
        support = (int) Math.pow(2, r);
        
        offset = new int [3];
        
        for ( int i=0; i < offset.length; i++ ) {
            offset[i] = support - 1;
        }
        
        wavelet = new HaarWavelet (3, r);
        
        coords = new WaveletCoords ();
    }

    public DataBlock reduce (DataCollection inData) {
        return this.reduce ( inData, inData.getBounds() );
    }

    public DataBlock reduce (DataCollection inData, DataBlock outBlock) {
        return this.reduce (inData, inData.getBounds(), outBlock);
    }

    public DataBlock reduce (DataCollection inData, ISBounds inBounds) {
        
        ISBounds result = this.resultBounds(inBounds);
        
        DataBlock outBlock  = inData.createDataBlock( result );
        
        return this.reduce ( inData, inBounds, outBlock );
    }

    public DataBlock reduce (   DataCollection inData, 
                                ISBounds inBounds, DataBlock outBlock) 
   
    {
        wavelet.doSummary(inData, inBounds, outBlock);
        
        return outBlock;
    }

    public ISBounds resultBounds(ISBounds original) {
        // Get the original offset...
        int [] lower = original.copyLowerArray();

        // Get the size of the new data set...
        ISBounds result = this.resultSize(original);

        
        // Move the result base coords
        result = coords.lowResToBase(resolution, result);
        lower = coords.lowResToBase(0, lower);

        result.move(lower);
        
        // Wavelet results do not share the original lower bound coord.
        // Move by the wavelet offset.
        // result.move(offset);

        return result;
    }

    public ISBounds resultSize(ISBounds original) {
        int [] dims = new int [3];
        

        original.getDimensions(dims);

        for (int i=0; i < dims.length ; i++) 
            dims[i] = ( dims[i] / support );

        return new ISBounds (dims);
    }
    
    public ISBounds resultInData (ISBounds original) {
        
        int [] low = original.copyLowerArray();
        int [] up = original.copyUpperArray();
        
        for ( int i=0; i<low.length; i++ ) {
            low[i] /= support;
            up[i] /= support;
        }
        
        return new ISBounds (low, up);
    }

    public static void main ( String args [] )
    {
        int [] dims = {17,17,17};
        ISBounds bounds = new ISBounds (dims);
        RecordDescriptor rd = new RecordDescriptor ('f', 1);
        DataBlock block = new BasicBlock (bounds, rd);
        
        ISIterator values = new ISIterator (bounds);
        
        for ( float v=0 ; values.valid(); values.next(), v += 1.0 ) {
            block.setFloat(values, 0, v);
        }
        
        Reducer ud = new WaveletReducer (4);
        
        int [] resultDims = {1,1,1};
        ISBounds resultBounds = new ISBounds (resultDims);
        DataBlock result = new BasicBlock (resultBounds, rd);
        
        ud.reduce(block, result);
        
        ISIterator it = new ISIterator (bounds);
        Datum d = result.createDatum();
        
        for ( it.init(); it.valid(); it.next() ) {
            block.datum(d, it); 
            System.err.println( d.getFloat(0) );
        }
        
        it = new ISIterator (resultBounds);
        
        for ( it.init(); it.valid(); it.next() ) {
            result.datum(d, it); 
            
            float f = d.getFloat(0);
            
            System.err.println( f );
            assert f == f;
        }
        

    }

}
