#include <iostream>
#include <fstream>
#include <string>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <netdb.h>
#include <ctype.h>
#include <sys/time.h>
#include <unistd.h>

using namespace std;

hostent* server;
sockaddr_in srvr_addr;
char buffer[1024];
int sd;

void write_sock(float*, int);
void read_sock();
void error(string, int);

int main (int argc, char* argv[]) {
	char* run_name = argv[0];
	if (argc < 3) {
		string run_name_str = run_name;
		string err = "Usage: " + run_name_str + " port filename";
		error(err, 1);
		cerr << "Usage: " << run_name << " port filename\n";
		exit(1);
	}
	int port = atoi(argv[1]);

	ifstream in;
	const char* filename = argv[2];
	char* host = "127.0.0.1";

	in.open(filename, ios::binary);
	
	if(!in) {
		error("Could not open input file!  Aborting...", 2);
	}


	sd = socket(AF_INET, SOCK_STREAM, 0);
	if(sd == -1) {
		error("Could not open socket!  Aborting...", 3);
	}

	srvr_addr.sin_family = AF_INET;
	server = gethostbyname(host);
	if(!server) {
		error ("Server not found!  Aborting...\n", 4);
		exit(-1);
	}
	srvr_addr.sin_port = htons(port);
	memmove ((char*) &srvr_addr.sin_addr.s_addr, (char*) server->h_addr_list[0], server->h_length);
	if(connect(sd, (sockaddr*) &srvr_addr, sizeof(srvr_addr)) == -1) {
		error("Could not connect!  Aborting...\n", 5);
		exit(-1);
	}


	long length;
	int buf_size = 32768;

	in.seekg(0, ios::end);
	length = in.tellg();
	in.seekg(0, ios::beg);

	float* data = new float[length];
	float* buffer = new float[buf_size];

	in.read((char*)data, length);
	timeval t1;
	gettimeofday(&t1, 0);

	long curPos = 0;
	int i = 0;		
	while(curPos < length) {
		for(i = 0; i < buf_size; i++, curPos++) {
			buffer[i] = data[curPos];
			if (curPos >= length)
				break;
		}
		if(curPos < length) {
			write_sock(buffer, buf_size);
		}
		else {
		//There will be 'i' elements in the final buffer
			float* endOfBuffer = new float[i];
			for(int j = 0; j < i; j++) {
				endOfBuffer[j] = buffer[j];
			}				
			write_sock(endOfBuffer, i);
			delete [] endOfBuffer;		
		}
		cout << "Wrote\n\n";
		read_sock();
		cout << "Read\n\n";
	}
	delete [] data;
	timeval t2;
	gettimeofday(&t2, 0);
	long seconds, useconds, mtime;
	seconds = t2.tv_sec - t1.tv_sec;
	useconds = t2.tv_usec - t1.tv_usec;
	mtime = ((seconds) * 1000 + useconds/1000.0) + 0.5;

	
	cout << "It took " << mtime << "ms to perform\n";
	close(sd);


	return 0;
}

void error(string value, int code) {
	perror(value.c_str());
	cout << "Program Error Code: " << code << "\n";
}

void read_sock() {
	int r_code;
	r_code = read(sd, buffer, 1023);
	if(r_code < 0) {
		error("Error reading from the socket",6);
	}
	cout << buffer << "\n";
}

void write_sock(float* message, int length) {
	int w_code;
	w_code = write(sd, message, length);
	if(w_code < 0) {
		error("Error writing to the socket", 7);
	}
}
