/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package selection;

import edu.unh.sdb.common.Datum;
import edu.unh.sdb.common.RecordDescriptor;
import edu.unh.sdb.datasource.BasicBlock;
import edu.unh.sdb.datasource.DataBlock;
import edu.unh.sdb.datasource.ISBounds;
import edu.unh.sdb.datasource.ISIterator;

/**
 *
 * @author svohr
 */
public class ErrorRegionInterpolator {
    
    private int regionSize;
    private Mapper indexMapper;
    private float [] buffer;
    
    
    public ErrorRegionInterpolator ()
    {
        indexMapper = new ISMapper ();
        buffer = new float [8];
    }
    
    public ErrorRegionInterpolator (Mapper m)
    {
        indexMapper = m;
        buffer = new float [8];
    }
    
    /**
     * Returns the average of two floats, i.e. the value directly between them.
     * @param a 
     * @param b
     * @return the average of a and b
     */
    private float interpolateBetween (float a, float b)
    {
        return (a + b) / 2;
    }
    
    /**
     * Returns the average of two floats, i.e. the value directly between them.
     * @param a 
     * @param b
     * @return the average of a and b
     */
    private float interpolateBetween (float a, float b, float fraction)
    {
        return a + ( (b - a) * fraction);
    }

    /**
     * Iterpolates a number of points in between the two given floats.
     * 
     * @param a
     * @param b
     * @param level The resolution level (0 is base resolution)
     * 
     * @return an array of floats containing a and b and all points in between.
     */
    private float [] interpolateAllBetween (float a, float b, int level)
    {
        float [] res = new float [ (int)(Math.pow(2, level)) + 1 ]; 
        
        res[0] = a;
        res[(res.length - 1)] = b;
        
        if ( level == 1 ) {
            res[1] = interpolateBetween (a,b);
        }
        else {
            float mid = interpolateBetween (a,b);
            int midPoint = res.length / 2;
            res[midPoint] = mid;
            
            float [] left = interpolateAllBetween ( a, mid, (level -1) );
            float [] right = interpolateAllBetween ( mid, b, (level -1) );
            
            for (int i = 0; i < midPoint; i++) {
                res[i] = left[i];
                res[midPoint + i] = right[i];
            }
        }
        
        /*
        System.err.print(Float.toString(a) + " " + Float.toString(b) + ">> ");
        for (int i=0; i<res.length; i++) {
            System.err.print (Float.toString(res[i]) + " ");
        }
        System.err.println(); */
        
        return res;
    }

    /**
     * Interpolates and fills in all points along the X axis starting with
     * the specified coords.
     * 
     * @param cell error cell
     * @param x1
     * @param y1
     * @param z1
     */
    private void reconstructX (float [][][] cell, int x1, int y1, int z1)
    {
        int len = regionSize;
        float a = cell[z1][y1][x1];
        float b = cell[z1][y1][len - 1];
        
    //  float [] row = interpolateAllBetween (cell[z1][y1][x1], 
    //          cell[z1][y1][len - 1], resLevel); 
        
        // copy everything
        for (int i=1; i < len-1; i++)    
            cell[z1][y1][i] = interpolateBetween( a, b, ((float)(i) / (len-1)) );
    //      cell[z1][y1][i] = row[i];
    }
    
        /**
     * Interpolates and fills in all points along the X axis starting with
     * the specified coords.
     * 
     * @param cell error cell
     * @param x1
     * @param y1
     * @param z1
     * @param x2
     * @param y2
     * @param z2
     */
    private void reconstructX ( float [][][] cell, int x1, int y1, int z1,
                                                   int x2, int y2, int z2)
    {
        int len = x2 - x1;
        float a = cell[z1][y1][x1];
        float b = cell[z1][y1][x2];
        
        // copy everything
        for (int i=1; i < len; i++)    
            cell[z1][y1][x1+i] = interpolateBetween( a, b, ((float)(i) / len) );
    }
    
    /**
     * Interpolates and fills in all points along the Y axis starting with
     * the specified coords.
     * 
     * @param cell error cell
     * @param x1
     * @param y1
     * @param z1
     */
    private void reconstructY (float [][][] cell, int x, int y, int z)
    {
        int len = regionSize;
        float a = cell[z][y][x];
        float b = cell[z][len - 1][x];
        
    //  float [] row = interpolateAllBetween (cell[z1][y1][x1], 
    //          cell[z1][len-1][x1], resLevel); 
        
        // copy everything
        for (int i=1; i < len-1; i++) 
            cell[z][i][x] = interpolateBetween( a, b, ((float)(i) / (len-1)) );
    //      cell[z1][i][x1] = row[i];
    }
    
        /**
     * Interpolates and fills in all points along the Y axis starting with
     * the specified coords.
     * 
     * @param cell error cell
     * @param x1
     * @param y1
     * @param z1
     * @param x2
     * @param y2
     * @param z2
     */
    private void reconstructY (float [][][] cell, int x1, int y1, int z1,
                                                  int x2, int y2, int z2)
    {
        int len = y2 - y1;
        float a = cell[z1][y1][x1];
        float b = cell[z1][y2][x1];
        
        // copy everything
        for (int i=1; i < len; i++) 
            cell[z1][y1+i][x1] = interpolateBetween( a, b, ((float)(i) / (len)) );
    }

    /**
     * Interpolates and fills in all points along the Z axis starting with
     * the specified coords.
     * 
     * @param cell error cell
     * @param x1
     * @param y1
     * @param z1
     */
    private void reconstructZ (float [][][] cell, int x, int y, int z)
    {
        int len = regionSize;
        float a = cell[z][y][x];
        float b = cell[len-1][y][x];
        
    //  float [] row = interpolateAllBetween (cell[z1][y1][x1], 
    //          cell[len-1][y1][x1], resLevel); 
        
        // copy everything
        for (int i=1; i < len-1; i++) 
            cell[i][y][x] = interpolateBetween( a, b, ((float)(i) / (len-1)) );
    //      cell[i][y1][x1] = row[i];
    }
    
    /**
     * Interpolates and fills in all points along the Z axis starting with
     * the specified coords.
     * 
     * @param cell error cell
     * @param x1
     * @param y1
     * @param z1
     */
    private void reconstructZ (float [][][] cell, int x1, int y1, int z1,
                                                  int x2, int y2, int z2)
    {
        int len = z2 - z1;
        float a = cell[z1][y1][x1];
        float b = cell[z2][y1][x1];
        
        // copy everything
        for (int i=1; i < len; i++) 
            cell[z1+i][y1][x1] = interpolateBetween( a, b, ((float)(i) / (len)) );
    }
    
    /**
     * Builds a reconstructed error cell from an actual cell.
     * @param actual the original error cell
     * @return the reconstructed error cell
     */
    public float [][][] interpolate (float [][][] actual)
    {
        regionSize = actual.length;
        int zLen = regionSize;
        int yLen = regionSize;
        int xLen = regionSize;
        
        float result [][][] = new float [zLen][yLen][xLen];
        
        return result;
    }
    
    /**
     * Builds a reconstructed error cell from an actual cell.
     * @param actual the original error cell
     * @param result the resulting cell
     * @return the reconstructed error cell
     */
    public float [][][] interpolate (float [][][] actual, float [][][] result) 
    {
        regionSize = actual.length;
        int zLen = regionSize;
        int yLen = regionSize;
        int xLen = regionSize;
        
        
        // set up the corners ( the selected/decimated data points )
        result [0][0][0] = actual [0][0][0];
        result [0][0][xLen-1] = actual [0][0][xLen-1];
        
        result [0][yLen-1][0] = actual [0][yLen-1][0];
        result [0][yLen-1][xLen-1] = actual [0][yLen-1][xLen-1];
        
        result [zLen-1][0][0] = actual [zLen-1][0][0];
        result [zLen-1][0][xLen-1] = actual [zLen-1][0][xLen-1];
        
        result [zLen-1][yLen-1][0] = actual [zLen-1][yLen-1][0];
        result [zLen-1][yLen-1][xLen-1] = actual [zLen-1][yLen-1][xLen-1];
        
        // build the horizontal axis
        reconstructX (result, 0,0,0);
        reconstructX (result, 0,yLen-1,0);
        reconstructX (result, 0,0,zLen-1);
        reconstructX (result, 0,yLen-1,zLen-1);
        
        // fill int the first and last slice.
        for (int i=0; i < xLen; i++) {
            reconstructY (result, i,0,0);
            reconstructY (result, i,0,zLen-1);
        }
        
        // fill in the slices in between.
        for (int i=0; i < xLen; i++) {
            for (int j=0; j < yLen; j++) {
                reconstructZ (result, i,j,0);
            }
        }

/*        System.err.println ("************************");
        for (int i=0; i < zLen; i++) {
            for (int j=0; j < yLen; j++) {
                for (int k=0; k < xLen; k++) {
                    System.err.print (
                            Float.toString(actual[i][j][k]) + " " +
                            Float.toString(result[i][j][k]) + " " );
                }
                System.err.println();
            }
        } 
        System.err.println ("************************"); */
        
        return result;
    }
    
    /**
     * Builds a reconstructed error cell from an actual cell.
     * @param actual the original error cell
     * @param result the resulting cell
     * @return the reconstructed error cell
     */
    public float [][][] interpolate (float [][][] actual, float [][][] result,
                                     int x1, int y1, int z1, int x2, int y2, int z2) 
    {
        int zLen = z2 - z1 + 1;
        int yLen = y2 - y1 + 1;
        int xLen = x2 - x1 + 1;
        
        
        // set up the corners ( the selected/decimated data points )
        result [z1][y1][x1] = actual [z1][y1][x1];
        result [z1][y1][x2] = actual [z1][y1][x2];
        
        result [z1][y2][x1] = actual [z1][y2][x1];
        result [z1][y2][x2] = actual [z1][y2][x2];
        
        result [z2][y1][x1] = actual [z2][y1][x1];
        result [z2][y1][x2] = actual [z2][y1][x2];
        
        result [z2][y2][x1] = actual [z2][y2][x1];
        result [z2][y2][x2] = actual [z2][y2][x2];
        
        // build the horizontal axis
        reconstructX (result, x1,y1,z1, x2,y1,z1);
        reconstructX (result, x1,y2,z1, x2,y2,z1);
        reconstructX (result, x1,y1,z2, x2,y1,z2);
        reconstructX (result, x1,y2,z2, x2,y2,z2);
        
        // fill int the first and last slice.
        for (int i=0; i < xLen; i++) {
            reconstructY (result, x1+i,y1,z1,x1+i,y2,z1);
            reconstructY (result, x1+i,y1,z2,x1+i,y2,z2);
        }
        
        // fill in the slices in between.
        for (int i=0; i < xLen; i++) {
            for (int j=0; j < yLen; j++) {
                reconstructZ (result, x1+i,y1+j,z1,x1+i,y1+i,z2);
            }
        }

/*        System.err.println ("************************");
        for (int i=0; i < zLen; i++) {
            for (int j=0; j < yLen; j++) {
                for (int k=0; k < xLen; k++) {
                    System.err.print (
                            Float.toString(actual[i][j][k]) + " " +
                            Float.toString(result[i][j][k]) + " " );
                }
                System.err.println();
            }
        } 
        System.err.println ("************************"); */
        
        return result;
    }
    
    
    public float interpolate1d (float v1, float p1, 
                                float v2, float p2, float p3)
    {
        /*
         * p1 -----------p3---------------- p2
         */
        float result =  (v1 * ( (p2 - p3) / (p2 - p1) ) ) +
                        (v2 * ( (p3 - p1) / (p2 - p1) ) );
        // System.err.printf ("%f (%f) --- %f (%f) --- %f (%f)\n",
        //                    v1, p1, result, p3, v2, p2);
        return result;

    }
    
    public float interpolate (DataBlock values, ISBounds base, int [] point)
    {
        int [] index = {0, 0, 0};
        

        // interpolate
        float [] pCoords = indexMapper.map (point);

        base.copyLowerArray(index);
        float [] lCoords = indexMapper.map (index);
        base.copyUpperArray(index);
        float [] uCoords = indexMapper.map (index);

        // this should be 8 values; one for each corner
        float [] vals = values.getFloats();
        float [] corners;


        // check if we have all 8 values.
        if ( vals.length == 8 ) {
            corners = vals;
        }
        else if ( vals.length == 1 ) {
            // Only 1 value. No interpolation necessary.
            return vals[0];
        }
        else {
            // must fill in missing values.
            corners = buffer;

            // number of values is 4.    
            int [] dims = values.getDimensionsArray();

            if ( dims[2] == 1 ) {
                for (int i=0; i<corners.length; i++)
                    corners[i] = vals[i%4];
            }
            else if ( dims[1] == 1 ) {
                for (int i=0; i<corners.length; i++) 
                    corners[i] = vals[2*(i/4) + (i%2)];
            }
            else { // if ( dims[0] == 1 )
                for (int i=0; i<corners.length; i++) 
                    corners[i] = vals[i/2];
            }
        }
        
        // values for interpolation
        float v00x, v01x, v10x, v11x;
        float v0yx, v1yx, vzyx;

        // do all the interpolation
        v00x = interpolate1d (  corners[0], lCoords[0], 
                                corners[1], uCoords[0], pCoords[0] );
        v01x = interpolate1d (  corners[2], lCoords[0], 
                                corners[3], uCoords[0], pCoords[0] );
        v10x = interpolate1d (  corners[4], lCoords[0], 
                                corners[5], uCoords[0], pCoords[0] );
        v11x = interpolate1d (  corners[6], lCoords[0], 
                                corners[7], uCoords[0], pCoords[0] );

        v0yx = interpolate1d (  v00x, lCoords[1],
                                v01x, uCoords[1], pCoords[1] );
        v1yx = interpolate1d (  v10x, lCoords[1],
                                v11x, uCoords[1], pCoords[1] );

        vzyx = interpolate1d (  v0yx, lCoords[2],
                                v1yx, uCoords[2], pCoords[2] );

        return vzyx;
    }
    
    public static void main( String[] args )
    {
        // Test cases
        int [] dims = {2,2,2};
        ISBounds bounds = new ISBounds (dims);
        RecordDescriptor rd = new RecordDescriptor ('f', 1);
        DataBlock block = new BasicBlock (bounds, rd);
        
        ISIterator values = new ISIterator (bounds);
        
        for ( float v=0 ; values.valid(); values.next(), v += 1.0 ) {
            block.setFloat(values, 0, v);
        }
        
        int [] baseDims = {10,20,30};
        ISBounds base = new ISBounds (baseDims);
        int [] point = {3,17,19};
        ErrorRegionInterpolator e = new ErrorRegionInterpolator ();
        
        float result = e.interpolate(block, base, point);
        
        System.out.println(result);
        
        return;
    }
    
}
