/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package selection;

import edu.unh.sdb.datasource.FDLDescriptor;
import edu.unh.sdb.datasource.ISBounds;
import java.io.FileNotFoundException;

/**
 *
 * @author svohr
 */
public class StdDevErrorWriter extends ErrorWriter {
    
    public StdDevErrorWriter (  String name, int dataLevel, int errorLevel,
                                int max, ISBounds size, FDLDescriptor base) 
            throws FileNotFoundException
    {
        super (name + "_stddev", dataLevel, errorLevel, max, size, base);
    }
    
    /**
     * Calculates the maximum absolute error between an actual and 
     * reconstructed error cell.
     * 
     * @param actual actual error cell data
     * @param reconstructed reconstructed error cell data
     * 
     * @return maximum absolute error of the cell
     */
    public float calculate (float [][][] actual, float [][][] reconstructed) 
    {
        int regionSize = actual.length;
        float count;
        float total;
        float mean;
        float result;
        
        // remember: the last row or column is not a part of the error cell.
        // hence length - 1.
        
        count = (float) Math.pow( regionSize - 1, 3);
        total = 0;
        
     // System.err.println ("---------------------- " + cellSize + "^3");
        
        // Calculate the mean value.
        
        for ( int z=0; z < regionSize-1; z++ ) {
            for ( int y=0; y < regionSize-1; y++ ) {
                for ( int x=0; x < regionSize-1; x++ ) {
                    
                    float error = Math.abs( 
                            actual[z][y][x] - reconstructed [z][y][x] );
                    
                    total += error; 
                    
                }
            }
        }
        
        mean = total / count;
        
        // Calculate the standard deviation
        for ( int z=0; z < regionSize-1; z++ ) {
            for ( int y=0; y < regionSize-1; y++ ) {
                for ( int x=0; x < regionSize-1; x++ ) {
                    
                    float error = (float) Math.pow( 
                           ( Math.abs ( actual[z][y][x] - 
                                        reconstructed [z][y][x] ) - mean), 2);
                    
                    total += error; 
                    
                }
            }
        }
        
        result = (float) Math.sqrt( total / count );
        
        
        return result;
    }
    
}
