/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package selection;

import edu.unh.sdb.common.Datum;
import edu.unh.sdb.datasource.DataBlock;
import edu.unh.sdb.datasource.DataCollection;
import edu.unh.sdb.datasource.ISBounds;
import edu.unh.sdb.datasource.ISIterator;

/**
 *
 * @author svohr
 */
public class UniformDecimation 
{
    
    private int interval;
    
    public UniformDecimation () 
    {
        // default is first resolution.
        this.interval = 2;
    }
    
    public UniformDecimation ( int resolution ) 
    {
        this.interval = (int)Math.pow(2, resolution);
    }
    
    public DataBlock decimate ( DataCollection inData ) 
    {
        // Make a DataBlock to return the summary data

        ISBounds inBounds = inData.getBounds();
        
        return this.decimate(inData, inBounds);

    }
    
    public DataBlock decimate ( DataCollection inData, ISBounds inBounds )
    {
        // need output bounds to be 1/2 size of the input in all dimensions
        // first get the sizes of the input
        int[]  outSizes = inBounds.getDimensionsArray();

        // now halve them -- It's not really half...
        for ( int i = 0; i < inData.dim(); i++ )
            outSizes[i] = (outSizes[i] + (interval-1)) / interval;
        // now make a bounds for the output
        ISBounds outBounds = new ISBounds( outSizes );

        // get an output block with the right bounds and just one attribute
        DataBlock outBlock  = inData.createDataBlock( outBounds );

        this.decimate ( inData, inBounds, outBlock );

        return outBlock;
    }
    
    public DataBlock decimate ( DataCollection inData, ISBounds inBounds, 
                                DataBlock outBlock )
    {
        // need output bounds to be 1/2 size of the input in all dimensions
        // first get the sizes of the input
        ISBounds outBounds = outBlock.getBounds();

        // Build the step array.
        int [] skip = new int [ inData.dim() ];
        for (int i=0; i<skip.length; i++)
            skip[i] = interval;
        
        
        // We need a block iterator that will go through the data space, but it
        //   will always step by 2 in each dimension. Hence, we need a negative
        //   gaps parameter
        ISIterator inIter = new ISIterator( inBounds, skip );

        // We also need an iterator over the output space, but this is a simple
        // iterator that just identifies one point at a time (not a block).
        ISIterator outIter = new ISIterator( outBounds );

        Datum dat = inData.createDatum();
        
        // Get the sample points from the data set. 
        for ( ; inIter.valid(); inIter.next(), outIter.next() )
        {
            inData.datum(dat, inIter);
            outBlock.setFloats(outIter, dat.getFloats());
        }
        
        return outBlock;
    }
}
